# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin, CosPlugin, PluginOpt)


class Filesys(Plugin, DebianPlugin, UbuntuPlugin, CosPlugin):
    """Collects general information about the local filesystem(s) and mount
    points as well as optional information about EXT filesystems. Note that
    information specific filesystems such as XFS or ZFS is not collected by
    this plugin, as there are specific plugins for those filesystem types.

    This plugin will collect /etc/fstab as well as mount information within
    /proc/, and is responsible for the 'mount' and 'df' symlinks that appear
    in an sos archive's root.
    """

    short_desc = 'Local file systems'

    plugin_name = 'filesys'
    profiles = ('storage',)

    option_list = [
        PluginOpt('lsof', default=False,
                  desc='collect information on all open files'),
        PluginOpt('dumpe2fs', default=False, desc='dump filesystem info'),
        PluginOpt('frag', default=False,
                  desc='collect filesystem fragmentation status')
    ]

    def setup(self):
        self.add_copy_spec([
            "/proc/fs/",
            "/proc/mounts",
            "/proc/filesystems",
            "/proc/self/mounts",
            "/proc/self/mountinfo",
            "/proc/self/mountstats",
            "/proc/[0-9]*/mountinfo",
            "/etc/mtab",
            "/etc/fstab",
            "/run/mount/utab",
        ])
        self.add_cmd_output("mount -l", root_symlink="mount",
                            tags="mount")
        self.add_cmd_output("df -al -x autofs", root_symlink="df",
                            tags='df__al')
        self.add_cmd_output([
            "df -aliT -x autofs",
            "findmnt",
            "lslocks"
        ])

        self.add_forbidden_path([
            # cifs plugin
            '/proc/fs/cifs',
            # lustre plugin
            '/proc/fs/ldiskfs',
            '/proc/fs/lustre',
            # nfs plugin
            '/proc/fs/nfsd',
            '/proc/fs/nfsfs',
            # panfs (from Panasas company) provides statistics which can be
            # very large (100s of GB)
            '/proc/fs/panfs',
            # xfs plugin
            '/proc/fs/xfs'
        ])

        if self.get_option('lsof'):
            self.add_cmd_output("lsof -b +M -n -l -P", root_symlink="lsof",
                                priority=50)

        dumpe2fs_opts = '-h'
        if self.get_option('dumpe2fs'):
            dumpe2fs_opts = ''
        mounts = '/proc/mounts'
        ext_fs_regex = r"^(/dev/\S+).+ext[234]\s+"
        for dev in self.do_regex_find_all(ext_fs_regex, mounts):
            self.add_cmd_output(f"dumpe2fs {dumpe2fs_opts} {dev}",
                                tags="dumpe2fs_h")

            if self.get_option('frag'):
                self.add_cmd_output(f"e2freefrag {dev}", priority=100)

    def postproc(self):
        self.do_file_sub(
            "/etc/fstab",
            r"(password=)[^,\s]*",
            r"\1********"
        )

        # remove expected errors from lsof due to command formatting, but still
        # keep stderr so other errors are reported
        regex = (r"(lsof: (avoiding (.*?)|WARNING: can't stat\(\) (.*?))|"
                 "Output information may be incomplete.)\n")

        self.do_cmd_output_sub("lsof", regex, '')


class RedHatFilesys(Filesys, RedHatPlugin):

    def setup(self):
        super().setup()
        self.add_cmd_output('ls -ldZ /tmp')

# vim: set et ts=4 sw=4 :



# Copyright (c) 2012 Adam Stokes <adam.stokes@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, DebianPlugin, UbuntuPlugin


class Apport(Plugin, DebianPlugin, UbuntuPlugin):

    short_desc = 'Apport crash reporting tool'

    plugin_name = 'apport'
    profiles = ('debug',)

    def setup(self):
        if not self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/apport.log",
                "/var/log/apport.log.1"
            ])
        else:
            self.add_copy_spec("/var/log/apport*")
            self.add_copy_spec("/var/crash/**")
        self.add_copy_spec("/etc/apport/*")
        self.add_copy_spec("/var/lib/whoopsie/whoopsie-id")
        self.add_cmd_output(
            "gdbus call -y -d com.ubuntu.WhoopsiePreferences \
            -o /com/ubuntu/WhoopsiePreferences \
            -m com.ubuntu.WhoopsiePreferences.GetIdentifier")
        self.add_dir_listing('/var/crash', recursive=True)
        self.add_cmd_output("bash -c 'grep -B 50 -m 1 ProcMaps /var/crash/*'")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class Rpm(Plugin, RedHatPlugin):

    short_desc = 'RPM Package Manager'

    plugin_name = 'rpm'
    profiles = ('system', 'packagemanager')

    option_list = [
        PluginOpt('rpmq', default=True,
                  desc='query package information with rpm -q'),
        PluginOpt('rpmva', default=False, desc='verify all packages'),
        PluginOpt('rpmdb', default=False, desc='collect /var/lib/rpm')
    ]

    verify_packages = ('rpm',)

    def setup(self):
        self.add_copy_spec("/var/log/rpmpkgs")
        self.add_dir_listing('/var/lib/rpm', recursive=True)

        if self.get_option("rpmq"):
            rpmq = "rpm --nodigest -qa --qf=%s"
            # basic installed-rpms
            nvra = '"%-59{NVRA} %{INSTALLTIME:date}\n"'
            irpms = f"sh -c '{rpmq} | sort -V'" % nvra

            self.add_cmd_output(irpms, root_symlink='installed-rpms',
                                tags='installed_rpms')

            # extended package data
            extpd = (
                '"%{NAME}-%{VERSION}-%{RELEASE}.%{ARCH}\\t'
                '%{INSTALLTIME:date}\\t%{INSTALLTIME}\\t'
                '%{VENDOR}\\t%{BUILDHOST}\\t'
                '%{SIGPGP}\\t%{SIGPGP:pgpsig}\\n"'
            )

            self.add_cmd_output(rpmq % extpd, suggest_filename='package-data',
                                tags=['installed_rpms', 'package_data'])

        if self.get_option("rpmva"):
            self.plugin_timeout = 1000
            self.add_cmd_output("rpm -Va", root_symlink="rpm-Va",
                                timeout=900, priority=100,
                                tags=['rpm_va', 'rpm_V', 'rpm_v',
                                      'rpm_V_packages'])

        if self.get_option("rpmdb"):
            self.add_cmd_output("lsof +D /var/lib/rpm",
                                suggest_filename='lsof_D_var_lib_rpm')
            self.add_copy_spec("/var/lib/rpm")

        self.add_cmd_output("rpm --showrc")

# vim: set et ts=4 sw=4 :



# Copyright (C) Steve Conklin <sconklin@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import sys
import re

from sos.report.plugins import RedHatPlugin
from sos.presets.redhat import (RHEL_PRESETS, RHV, RHEL, CB, RHOSP,
                                RHOCP, RH_CFME, RH_SATELLITE, AAPEDA,
                                AAPCONTROLLER)
from sos.policies.distros import LinuxPolicy, ENV_HOST_SYSROOT, OS_RELEASE
from sos.policies.package_managers.rpm import RpmPackageManager
from sos.policies.package_managers.flatpak import FlatpakPackageManager
from sos.policies.package_managers import MultiPackageManager
from sos.utilities import bold
from sos import _sos as _


RHEL_RELEASE_STR = "Red Hat Enterprise Linux"


class RedHatPolicy(LinuxPolicy):
    vendor = "Red Hat"
    vendor_urls = [
        ('Distribution Website', 'https://www.redhat.com/'),
        ('Commercial Support', 'https://access.redhat.com/')
    ]
    _tmp_dir = "/var/tmp"
    _in_container = False
    name_pattern = 'friendly'
    upload_url = None
    upload_user = None
    default_container_runtime = 'podman'
    sos_pkg_name = 'sos'
    sos_bin_path = '/usr/sbin'
    client_identifier_url = "https://sso.redhat.com/auth/"\
        "realms/redhat-external/protocol/openid-connect/auth/device"
    token_endpoint = "https://sso.redhat.com/auth/realms/"\
        "redhat-external/protocol/openid-connect/token"

    def __init__(self, sysroot=None, init=None, probe_runtime=True,
                 remote_exec=None):
        super().__init__(sysroot=sysroot, init=init,
                         probe_runtime=probe_runtime,
                         remote_exec=remote_exec)
        self.usrmove = False

        self.package_manager = MultiPackageManager(
                primary=RpmPackageManager,
                fallbacks=[FlatpakPackageManager],
                chroot=self.sysroot,
                remote_exec=remote_exec)

        self.valid_subclasses += [RedHatPlugin]

        self.pkgs = self.package_manager.packages

        # If rpm query failed, exit
        if not self.pkgs:
            sys.stderr.write("Could not obtain installed package list")
            sys.exit(1)

        self.usrmove = self.check_usrmove(self.pkgs)

        if self.usrmove:
            self.PATH = "/usr/sbin:/usr/bin:/root/bin"
        else:
            self.PATH = "/sbin:/bin:/usr/sbin:/usr/bin:/root/bin"
        self.PATH += os.pathsep + "/usr/local/bin"
        self.PATH += os.pathsep + "/usr/local/sbin"
        if not self.remote_exec:
            self.set_exec_path()
        self.load_presets()

    @classmethod
    def display_distro_help(cls, section):
        if cls is not RedHatPolicy:
            super(RedHatPolicy, cls).display_distro_help(section)
            return
        section.add_text(
            'This policy is a building block for all other Red Hat family '
            'distributions. You are likely looking for one of the '
            'distributions listed below.\n'
        )

        subs = {
            'centos': CentOsPolicy,
            'rhel': RHELPolicy,
            'redhatcoreos': RedHatCoreOSPolicy,
            'fedora': FedoraPolicy
        }

        for subc, value in subs.items():
            subln = bold(f"policies.{subc}")
            section.add_text(
                f"{' ':>8}{subln:<35}{value.os_release_name:<30}",
                newline=False
            )

    def check_usrmove(self, pkgs):
        """Test whether the running system implements UsrMove.

            If the 'filesystem' package is present, it will check that the
            version is greater than 3. If the package is not present the
            '/bin' and '/sbin' paths are checked and UsrMove is assumed
            if both are symbolic links.

            :param pkgs: a packages dictionary
        """
        if 'filesystem' not in pkgs:
            return os.path.islink('/bin') and os.path.islink('/sbin')
        filesys_version = pkgs['filesystem']['version']
        return filesys_version[0] == '3'

    def mangle_package_path(self, files):
        """Mangle paths for post-UsrMove systems.

            If the system implements UsrMove, all files will be in
            '/usr/[s]bin'. This method substitutes all the /[s]bin
            references in the 'files' list with '/usr/[s]bin'.

            :param files: the list of package managed files
        """
        paths = []

        def transform_path(path):
            # Some packages actually own paths in /bin: in this case,
            # duplicate the path as both the / and /usr version.
            skip_paths = ["/bin/rpm", "/bin/mailx"]
            if path in skip_paths:
                return (path, os.path.join("/usr", path[1:]))
            return (re.sub(r'(^)(/s?bin)', r'\1/usr\2', path),)

        if self.usrmove:
            for f in files:
                paths.extend(transform_path(f))
            return paths
        return files

    def get_tmp_dir(self, opt_tmp_dir):
        if not opt_tmp_dir:
            return self._tmp_dir
        return opt_tmp_dir


# Legal disclaimer text for Red Hat products
disclaimer_text = """
Any information provided to %(vendor)s will be treated in \
accordance with the published support policies at:\n
  %(vendor_urls)s

The generated archive may contain data considered sensitive \
and its content should be reviewed by the originating \
organization before being passed to any third party.

No changes will be made to system configuration.
"""

RH_API_HOST = "https://api.access.redhat.com"
RH_SFTP_HOST = "sftp://sftp.access.redhat.com"


class RHELPolicy(RedHatPolicy):
    """
    The RHEL policy is used specifically for Red Hat Enterprise Linux, of
    any release, and not forks or derivative distributions. For example, this
    policy will be loaded for any RHEL 8 installation, but will not be loaded
    for CentOS Stream 8 or Red Hat CoreOS, for which there are separate
    policies.

    Plugins activated by installed packages will only be activated if those
    packages are installed via RPM (dnf/yum inclusive). Packages installed by
    other means are not considered by this policy.

    By default, --upload will be directed to using the SFTP location provided
    by Red Hat for technical support cases. Users who provide login credentials
    for their Red Hat Customer Portal account will have their archives uploaded
    to a user-specific directory.

    If users provide those credentials as well as a case number, --upload will
    instead attempt to directly upload archives to the referenced case, thus
    streamlining the process of providing data to technical support engineers.

    If either or both of the credentials or case number are omitted or are
    incorrect, then a temporary anonymous user will be used for upload to the
    SFTP server, and users will need to provide that information to their
    technical support engineer. This information will be printed at the end of
    the upload process for any sos report execution.
    """
    vendor = "Red Hat"
    os_release_file = '/etc/redhat-release'
    os_release_name = RHEL_RELEASE_STR
    os_release_id = 'rhel'
    msg = _("""\
This command will collect diagnostic and configuration \
information from this %(os_release_name)s system and installed \
applications.

An archive containing the collected information will be \
generated in %(tmpdir)s and may be provided to a %(vendor)s \
support representative.
""" + disclaimer_text + "%(vendor_text)s\n")
    _upload_url = RH_SFTP_HOST
    _upload_method = 'post'
    _device_token = None
    # Max size for an http single request is 1Gb
    _max_size_request = 1073741824

    def __init__(self, sysroot=None, init=None, probe_runtime=True,
                 remote_exec=None):
        super().__init__(sysroot=sysroot, init=init,
                         probe_runtime=probe_runtime,
                         remote_exec=remote_exec)
        self.register_presets(RHEL_PRESETS)

    def dist_version(self):
        try:
            rr = self.package_manager.all_pkgs_by_name_regex("redhat-release*")
            pkgname = self.pkgs[rr[0]]["version"]
            # this should always map to the major version number. This will not
            # be so on RHEL 5, but RHEL 5 does not support python3 and thus
            # should never run a version of sos with this check
            return int(pkgname[0])
        except Exception:
            pass
        return False

    def probe_preset(self):
        # Emergency or rescue mode?
        for target in ["rescue", "emergency"]:
            if self.init_system.is_running(f"{target}.target", False):
                return self.find_preset(CB)
        # Package based checks
        if self.pkg_by_name("satellite-common") is not None:
            return self.find_preset(RH_SATELLITE)
        if self.pkg_by_name("rhosp-release") is not None:
            return self.find_preset(RHOSP)
        if self.pkg_by_name("cfme") is not None:
            return self.find_preset(RH_CFME)
        if self.pkg_by_name("ovirt-engine") is not None or \
                self.pkg_by_name("vdsm") is not None:
            return self.find_preset(RHV)
        if self.pkg_by_name("automation-controller-server") is not None:
            return self.find_preset(AAPCONTROLLER)
        for pkg in ['automation-eda-controller',
                    'automation-eda-controller-server']:
            if self.pkg_by_name(pkg) is not None:
                return self.find_preset(AAPEDA)

        # Vanilla RHEL is default
        return self.find_preset(RHEL)


class CentOsPolicy(RHELPolicy):
    vendor = "CentOS"
    vendor_urls = [('Community Website', 'https://www.centos.org/')]
    os_release_file = '/etc/centos-release'
    os_release_name = 'CentOS Linux'
    os_release_id = 'centos'


class RedHatCoreOSPolicy(RHELPolicy):
    """
    Red Hat CoreOS is a containerized host built upon Red Hat Enterprise Linux
    and as such this policy is built on top of the RHEL policy. For users, this
    should be entirely transparent as any behavior exhibited or influenced on
    RHEL systems by that policy will be seen on RHCOS systems as well.

    The one change is that this policy ensures that sos collect will deploy a
    container on RHCOS systems in order to facilitate sos report collection,
    as RHCOS discourages non-default package installation via rpm-ostree which
    is used to maintain atomicity for RHCOS nodes. The default container image
    used by this policy is the support-tools image maintained by Red Hat on
    registry.redhat.io.

    Note that this policy is only loaded when sos is directly run on an RHCOS
    node - if sos collect uses the `oc` transport (the default transport that
    will be attempted by the ocp cluster profile), then the policy loaded
    inside the launched pod will be RHEL. Again, this is expected and will not
    impact how sos report collections are performed.
    """

    os_release_name = "Red Hat Enterprise Linux CoreOS"
    msg = _("""\
This command will collect diagnostic and configuration \
information from this %(os_release_name)s system.

An archive containing the collected information will be \
generated in %(tmpdir)s and may be provided to a %(vendor)s \
support representative.
""" + disclaimer_text + "%(vendor_text)s\n")

    containerized = True
    container_runtime = 'podman'
    container_image = 'registry.redhat.io/rhel8/support-tools'
    sos_path_strip = '/host'
    container_version_command = 'rpm -q sos'
    container_authfile = '/var/lib/kubelet/config.json'

    def __init__(self, sysroot=None, init=None, probe_runtime=True,
                 remote_exec=None):
        super().__init__(sysroot=sysroot, init=init,
                         probe_runtime=probe_runtime,
                         remote_exec=remote_exec)

    @classmethod
    def check(cls, remote=''):

        if remote:
            return 'CoreOS' in remote

        coreos = False
        if ENV_HOST_SYSROOT not in os.environ:
            return coreos
        host_release = os.environ[ENV_HOST_SYSROOT] + OS_RELEASE
        try:
            with open(host_release, 'r', encoding='utf-8') as hfile:
                for line in hfile.read().splitlines():
                    coreos |= cls.os_release_name in line
        except IOError:
            # host release file not present, will fallback to RHEL policy check
            pass
        return coreos

    def probe_preset(self):
        # As of the creation of this policy, RHCOS is only available for
        # RH OCP environments.
        return self.find_preset(RHOCP)

    def create_sos_container(self, image=None, auth=None, force_pull=False):
        _image = image or self.container_image
        _pull = '--pull=always' if force_pull else ''
        return (
            f"{self.container_runtime} run -di "
            f"--name {self.sos_container_name} --privileged --ipc=host "
            f"--net=host --pid=host -e HOST=/host "
            f"-e NAME={self.sos_container_name} -e "
            f"IMAGE={_image} {_pull} "
            f"-v /run:/run -v /var/log:/var/log "
            f"-v /etc/machine-id:/etc/machine-id "
            f"-v /etc/localtime:/etc/localtime "
            f"-v /:/host "
            f"{auth or ''} {_image}"
        )

    def set_cleanup_cmd(self):
        return f'podman rm --force {self.sos_container_name}'


class FedoraPolicy(RedHatPolicy):
    """
    The policy for Fedora based systems, regardless of spin/edition. This
    policy is based on the parent Red Hat policy, and thus will only check for
    RPM packages when considering packaged-based plugin enablement. Packages
    installed by other sources are not considered.

    There is no default --upload location for this policy. If users need to
    upload an sos report archive from a Fedora system, they will need to
    provide the location via --upload-url, and optionally login credentials
    for that location via --upload-user and --upload-pass (or the appropriate
    environment variables).
    """
    vendor = "the Fedora Project"
    vendor_urls = [
        ('Community Website', 'https://fedoraproject.org/'),
        ('Community Forums', 'https://discussion.fedoraproject.org/')
    ]
    os_release_file = '/etc/fedora-release'
    os_release_name = 'Fedora Linux'
    os_release_id = 'fedora'
    sos_bin_path = '/usr/bin'

    def __init__(self, sysroot=None, init=None, probe_runtime=True,
                 remote_exec=None):
        super().__init__(sysroot=sysroot, init=init,
                         probe_runtime=probe_runtime,
                         remote_exec=remote_exec)

    def fedora_version(self):
        pkg = self.pkg_by_name("fedora-release") or \
            self.package_manager.all_pkgs_by_name_regex(
                "fedora-release-.*")[-1]
        return int(pkg["version"])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import glob
from sos.policies.distros.redhat import RedHatPolicy
from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class Kernel(Plugin, IndependentPlugin):
    """The Kernel plugin is aimed at collecting general information about
    the locally running kernel. This information should be distribution-neutral
    using commands and filesystem collections that are ubiquitous across
    distributions.

    Debugging information from /sys/kernel/debug is collected by default,
    however care is taken so that these collections avoid areas like
    /sys/kernel/debug/tracing/trace_pipe which would otherwise cause the
    sos collection attempt to appear to 'hang'.

    The 'trace' option will enable the collection of the
    /sys/kernel/debug/tracing/trace file specfically, but will not change the
    behavior stated above otherwise.
    """

    short_desc = 'Linux kernel'

    plugin_name = 'kernel'
    profiles = ('system', 'hardware', 'kernel')
    verify_packages = ('kernel$',)

    sys_module = '/sys/module'

    option_list = [
        PluginOpt('with-timer', default=False,
                  desc='gather /proc/timer* statistics'),
        PluginOpt('trace', default=False,
                  desc='gather /sys/kernel/debug/tracing/trace file')
    ]

    def setup(self):
        # RedHat distributions can deliver kernel in RPM named either 'kernel'
        # or 'kernel-redhat', so we must verify both
        if isinstance(self.policy, RedHatPolicy):
            self.verify_packages = ('kernel$', 'kernel-redhat$')

        # compat
        self.add_cmd_output("uname -a", root_symlink="uname", tags="uname")
        self.add_cmd_output("lsmod", root_symlink="lsmod", tags="lsmod")
        self.add_dir_listing('/sys/kernel/slab')

        try:
            modules = self.listdir(self.sys_module)
            self.add_cmd_output("modinfo " + " ".join(modules),
                                suggest_filename="modinfo_ALL_MODULES",
                                tags='modinfo_all')
        except OSError:
            self._log_warn(f"could not list {self.sys_module}")

        # find /lib/modules/*/{extras,updates,weak-updates} -ls
        extra_mod_patterns = [
            "/lib/modules/*/extra",
            "/lib/modules/*/updates",
            "/lib/modules/*/weak-updates",
        ]
        extra_mod_paths = []
        for pattern in extra_mod_patterns:
            extra_mod_paths.extend(glob.glob(pattern))

        if extra_mod_paths:
            self.add_cmd_output(f"find {' '.join(extra_mod_paths)} -ls")

        self.add_cmd_output([
            "dmesg",
            "dmesg -T",
            "dkms status"
        ], cmd_as_tag=True)
        self.add_cmd_output("sysctl -a", tags="sysctl")

        clocksource_path = "/sys/devices/system/clocksource/clocksource0/"

        self.add_forbidden_path([
            '/sys/kernel/debug/tracing/trace_pipe',
            '/sys/kernel/debug/tracing/README',
            '/sys/kernel/debug/tracing/trace_stat',
            '/sys/kernel/debug/tracing/per_cpu',
            '/sys/kernel/debug/tracing/events',
            '/sys/kernel/debug/tracing/free_buffer',
            '/sys/kernel/debug/tracing/trace_marker',
            '/sys/kernel/debug/tracing/trace_marker_raw',
            '/sys/kernel/debug/tracing/instances/*/per_cpu/*/snapshot_raw',
            '/sys/kernel/debug/tracing/instances/*/per_cpu/*/trace_pipe*',
            '/sys/kernel/debug/tracing/instances/*/trace_pipe'
        ])

        self.add_copy_spec([
            "/proc/modules",
            "/proc/sys/kernel/random/boot_id",
            "/sys/module/*/parameters",
            "/sys/module/*/initstate",
            "/sys/module/*/refcnt",
            "/sys/module/*/taint",
            "/sys/module/*/version",
            "/sys/firmware/acpi/*",
            "/sys/kernel/debug/tracing/*",
            "/sys/kernel/livepatch/*",
            "/proc/kallsyms",
            "/proc/buddyinfo",
            "/proc/slabinfo",
            "/proc/zoneinfo",
            f"/lib/modules/{self.policy.kernel_version()}/modules.dep",
            "/etc/conf.modules",
            "/etc/modules.conf",
            "/etc/modprobe.conf",
            "/etc/modprobe.d",
            "/lib/modprobe.d",
            "/run/modprobe.d",
            "/usr/local/lib/modprobe.d",
            "/etc/sysctl.conf",
            "/etc/sysctl.d",
            "/lib/sysctl.d",
            "/proc/cmdline",
            "/proc/driver",
            "/proc/sys/kernel/tainted",
            "/proc/softirqs",
            "/proc/lock*",
            "/proc/misc",
            "/var/log/dmesg",
            "/sys/fs/pstore",
            "/var/lib/systemd/pstore",
            "/sys/kernel/hardlockup_count",
            "/sys/kernel/softlockup_count",
            "/sys/kernel/warn_count",
            "/sys/kernel/oops_count",
            "/sys/kernel/debug/dynamic_debug/control",
            "/sys/kernel/debug/extfrag/unusable_index",
            "/sys/kernel/debug/extfrag/extfrag_index",
            "/sys/kernel/debug/hv-balloon",
            clocksource_path + "available_clocksource",
            clocksource_path + "current_clocksource",
            "/proc/pressure/",
            f"/boot/config-{self.policy.kernel_version()}"
        ])

        if self.get_option("with-timer"):
            # This can be very slow, depending on the number of timers,
            # and may also cause softlockups
            self.add_copy_spec("/proc/timer*")

        if not self.get_option("trace"):
            self.add_forbidden_path("/sys/kernel/debug/tracing/trace")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class SANLock(Plugin):

    short_desc = 'SANlock daemon'
    plugin_name = "sanlock"
    profiles = ('cluster', 'virt')
    packages = ("sanlock",)

    def setup(self):
        self.add_copy_spec("/var/log/sanlock.log*")
        self.add_cmd_output([
            "sanlock client status -D",
            "sanlock client host_status -D",
            "sanlock client log_dump"
        ])


class RedHatSANLock(SANLock, RedHatPlugin):

    files = ("/etc/sysconfig/sanlock",)

    def setup(self):
        super().setup()
        self.add_copy_spec("/etc/sysconfig/sanlock")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import os
from sos.upload.targets import UploadTarget
from sos.policies.distros.ubuntu import UbuntuPolicy


class UbuntuUploadTarget(UploadTarget):

    upload_target_name = 'Ubuntu Upload Target'
    upload_target_id = "canonical"
    _upload_url = "https://files.support.canonical.com/uploads/"
    _upload_user = "ubuntu"
    _upload_password = "ubuntu"
    _upload_method = "put"

    def __init__(self, parser=None, args=None, cmdline=None):
        super().__init__(parser=parser, args=args, cmdline=cmdline)

    def check_distribution(self):
        """ Return true if we are running in a Ubuntu system"""
        return isinstance(self.commons['policy'], UbuntuPolicy)

    def get_upload_https_auth(self, user=None, password=None):
        if self.upload_url.startswith(self._upload_url):
            return (self._upload_user, self._upload_password)
        return super().get_upload_https_auth()

    def get_upload_url_string(self):
        if self.upload_url.startswith(self._upload_url):
            return "Canonical Support File Server"
        return self._get_obfuscated_upload_url(self.get_upload_url())

    def get_upload_url(self):
        if not self.upload_url or self.upload_url.startswith(self._upload_url):
            if not self.upload_archive_name:
                return self._upload_url
            fname = os.path.basename(self.upload_archive_name)
            return self._upload_url + fname
        return super().get_upload_url()

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, RedHatPlugin, UbuntuPlugin,
                                DebianPlugin, SoSPredicate, PluginOpt)
from sos.policies.distros.ubuntu import UbuntuPolicy
from sos.policies.distros.debian import DebianPolicy


class Networking(Plugin):

    short_desc = 'Network and networking devices configuration'

    plugin_name = "networking"
    profiles = ('network', 'hardware', 'system')
    trace_host = "www.example.com"

    option_list = [
        PluginOpt("traceroute", default=False,
                  desc=f"collect a traceroute to {trace_host}"),
        PluginOpt("namespace-pattern", default="", val_type=str,
                  desc=("Specific namespace names or patterns to collect, "
                        "whitespace delimited.")),
        PluginOpt("namespaces", default=None, val_type=int,
                  desc="Number of namespaces to collect, 0 for unlimited"),
        PluginOpt("ethtool-namespaces", default=True,
                  desc=("Toggle if ethtool commands should be run for each "
                        "namespace")),
        PluginOpt("eepromdump", default=False,
                  desc="Toggle collection of 'ethtool -e' for NICs")
    ]

    # switch to enable netstat "wide" (non-truncated) output mode
    ns_wide = "-W"

    # list of kernel modules needed by ss_cmd, this may vary by distro version
    ss_kmods = ['tcp_diag', 'udp_diag', 'inet_diag', 'unix_diag',
                'netlink_diag', 'af_packet_diag', 'xsk_diag']

    # list of ethtool short options, used in add_copy_spec and add_cmd_tags
    # do NOT add there "e" (see eepromdump plugopt)
    ethtool_shortopts = "acdgiklmPST"

    def setup(self):
        super().setup()

        self.add_file_tags({
            '/proc/net/bonding/bond.*': 'bond',
            '/etc/hosts': 'hosts'
        })

        self.add_copy_spec([
            "/etc/dnsmasq*",
            "/etc/host*",
            "/etc/inetd.conf",
            "/etc/iproute2",
            "/etc/network*",
            "/etc/nsswitch.conf",
            "/etc/resolv.conf",
            "/etc/gai.conf",
            "/etc/xinetd.conf",
            "/etc/xinetd.d",
            "/etc/yp.conf",
            "/proc/net/",
            "/sys/class/net/*/device/numa_node",
            "/sys/class/net/*/flags",
            "/sys/class/net/*/statistics/",
            "/etc/nmstate/",
            "/var/lib/lldpad/",
            "/etc/services",
        ])

        self.add_forbidden_path([
            "/proc/net/rpc/use-gss-proxy",
            "/proc/net/rpc/*/channel",
            "/proc/net/rpc/*/flush",
            # Cisco CDP
            "/proc/net/cdp",
            "/sys/net/cdp",
            # Dialogic Diva
            "/proc/net/eicon"
        ])

        self.add_cmd_output("ip -o addr", root_symlink="ip_addr",
                            tags='ip_addr')
        self.add_cmd_output("ip route show table all", root_symlink="ip_route",
                            tags=['ip_route', 'iproute_show_table_all'])
        self.add_cmd_output("plotnetcfg")

        self.add_cmd_output(f"netstat {self.ns_wide} -neopa",
                            root_symlink="netstat")

        self.add_cmd_output([
            "nstat -zas",
            "netstat -s",
            "netstat -s -6",
            f"netstat {self.ns_wide} -agn",
            "networkctl status -a",
            "ip -6 route show table all",
            "ip -d route show cache",
            "ip -d -6 route show cache",
            "ip -4 rule list",
            "ip -6 rule list",
            "ip vrf show",
            "ip -s -d link",
            "ip -d address",
            "ifenslave -a",
            "ip mroute show",
            "ip maddr show",
            "ip -s -s neigh show",
            "ip neigh show nud noarp",
            "biosdevname -d",
            "tc -s qdisc show",
            "nmstatectl show",
            "nmstatectl show --running-config",
        ])

        if self.path_isdir('/sys/class/devlink'):
            self.add_cmd_output([
                "devlink dev param show",
                "devlink dev info",
                "devlink port show",
                "devlink sb show",
                "devlink sb pool show",
                "devlink sb port pool show",
                "devlink sb tc bind show",
                "devlink -s -v trap show",
            ])

            devlinks = self.collect_cmd_output("devlink dev")
            if devlinks['status'] == 0:
                devlinks_list = devlinks['output'].splitlines()
                for devlink in devlinks_list:
                    self.add_cmd_output([
                        f"devlink dev eswitch show {devlink}",
                        f"devlink sb occupancy snapshot {devlink}",
                        f"devlink sb occupancy show {devlink}",
                        f"devlink -v resource show {devlink}"
                    ])
                    dev_tables = []
                    dpipe = self.collect_cmd_output(
                        f"devlink dpipe table show {devlink}"
                    )
                    if dpipe['status'] == 0:
                        for tableln in dpipe['output'].splitlines():
                            if tableln.startswith('name'):
                                dev_tables.append(tableln.split()[1])
                        self.add_cmd_output([
                            f"devlink dpipe table show {devlink} name {dname}"
                            for dname in dev_tables
                        ])

        # below commands require some kernel module(s) to be loaded
        # run them only if the modules are loaded, or if explicitly requested
        # via --allow-system-changes option
        ip_macsec_show_cmd = "ip -s macsec show"
        macsec_pred = SoSPredicate(self, kmods=['macsec'])
        self.add_cmd_output(ip_macsec_show_cmd, pred=macsec_pred, changes=True)

        self.collect_ss_ip_ethtool_info()
        self.collect_bridge_info()

    def add_command_tags(self):
        """ Command tags for ip/ethtool/netstat """
        for opt in self.ethtool_shortopts:
            self.add_cmd_tags({
                f'ethtool -{opt} .*': f'ethool_{opt}'
            })

        self.add_cmd_tags({
            "ethtool [^-].*": "ethtool",
            "ip -d address": "ip_addr",
            "ip -s -s neigh show": "ip_neigh_show",
            "ip -s -d link": "ip_s_link",
            "netstat.*-neopa": "netstat",
            "netstat.*-agn": "netstat_agn",
            "netstat -s": "netstat_s"
        })

    def collect_bridge_info(self):
        """ Collect information about bridges (some data already collected via
        "ip .." commands)
        """
        self.add_cmd_output([
            "bridge -s -s -d link show",
            "bridge -s -s -d -t fdb show",
            "bridge -s -s -d -t mdb show",
            "bridge -d vlan show"
        ])

    def collect_ss_ip_ethtool_info(self):
        """ Collect ss, ip and ethtool cmd outputs """
        ss_cmd = "ss -peaonmi"
        ss_pred = SoSPredicate(self, kmods=self.ss_kmods,
                               required={'kmods': 'all'})
        self.add_cmd_output(ss_cmd, pred=ss_pred, changes=True)

        self.add_cmd_output("ss -s")
        # Get ethtool output for every device that does not exist in a
        # namespace.
        _ecmds = [f"ethtool -{opt}" for opt in self.ethtool_shortopts]
        self.add_device_cmd([
            _cmd + " %(dev)s" for _cmd in _ecmds
        ], devices='ethernet')

        self.add_device_cmd([
            "ethtool %(dev)s",
            "ethtool --phy-statistics %(dev)s",
            "ethtool --show-priv-flags %(dev)s",
            "ethtool --show-eee %(dev)s",
            "ethtool --show-fec %(dev)s",
            "ethtool --show-ntuple %(dev)s",
            "tc -s filter show dev %(dev)s",
            "tc -s filter show dev %(dev)s ingress",
        ], devices="ethernet")

        # skip EEPROM collection by default, as it might hang or
        # negatively impact the system on some device types
        if self.get_option("eepromdump"):
            cmd = "ethtool -e %(dev)s"
            self._log_warn("WARNING: collecting an eeprom dump is known to "
                           "cause certain NIC drivers (e.g. bnx2x/tg3) to "
                           "interrupt device operation")
            self.add_device_cmd(cmd, devices="ethernet")

        if self.get_option("traceroute"):
            self.add_cmd_output(f"/bin/traceroute -n {self.trace_host}",
                                priority=100)

        # Capture additional data from namespaces; each command is run
        # per-namespace.
        self.add_cmd_output("ip netns")
        cmd_prefix = "ip netns exec "
        namespaces = self.get_network_namespaces(
                self.get_option("namespace-pattern"),
                self.get_option("namespaces"))
        if namespaces:
            # 'ip netns exec <foo> iptables-save' must be guarded by nf_tables
            # kmod, if 'iptables -V' output contains 'nf_tables'
            # analogously for ip6tables
            cout = {'cmd': 'iptables -V', 'output': 'nf_tables'}
            co6 = {'cmd': 'ip6tables -V', 'output': 'nf_tables'}
            iptables_with_nft = (SoSPredicate(self, kmods=['nf_tables'])
                                 if self.test_predicate(self,
                                 pred=SoSPredicate(self, cmd_outputs=cout))
                                 else None)
            ip6tables_with_nft = (SoSPredicate(self, kmods=['nf_tables'])
                                  if self.test_predicate(self,
                                  pred=SoSPredicate(self, cmd_outputs=co6))
                                  else None)

            for namespace in namespaces:
                _devs = self.devices['namespaced_network'][namespace]
                _subdir = f"namespaces/{namespace}"
                ns_cmd_prefix = cmd_prefix + namespace + " "
                self.add_cmd_output([
                    f"{ns_cmd_prefix} ip -d address show",
                    f"{ns_cmd_prefix} ip route show table all",
                    f"{ns_cmd_prefix} ip -s -s neigh show",
                    f"{ns_cmd_prefix} ip -4 rule list",
                    f"{ns_cmd_prefix} ip -6 rule list",
                    f"{ns_cmd_prefix} ip vrf show",
                    f"{ns_cmd_prefix} sysctl -a",
                    f"{ns_cmd_prefix} netstat {self.ns_wide} -neopa",
                    f"{ns_cmd_prefix} netstat -s",
                    f"{ns_cmd_prefix} netstat {self.ns_wide} -agn",
                    f"{ns_cmd_prefix} nstat -zas",
                ], priority=50, subdir=_subdir)
                self.add_cmd_output([ns_cmd_prefix + "iptables-save"],
                                    pred=iptables_with_nft,
                                    subdir=_subdir,
                                    priority=50)
                self.add_cmd_output([ns_cmd_prefix + "ip6tables-save"],
                                    pred=ip6tables_with_nft,
                                    subdir=_subdir,
                                    priority=50)

                ss_cmd = ns_cmd_prefix + "ss -peaonmi"
                # --allow-system-changes is handled directly in predicate
                # evaluation, so plugin code does not need to separately
                # check for it
                self.add_cmd_output(ss_cmd, pred=ss_pred, subdir=_subdir)

                # Collect ethtool commands only when ethtool_namespaces
                # is set to true.
                if self.get_option("ethtool-namespaces"):
                    # Devices that exist in a namespace use less ethtool
                    # parameters. Run this per namespace.
                    self.add_device_cmd([
                        f"{ns_cmd_prefix} ethtool %(dev)s",
                        f"{ns_cmd_prefix} ethtool -i %(dev)s",
                        f"{ns_cmd_prefix} ethtool -k %(dev)s",
                        f"{ns_cmd_prefix} ethtool -S %(dev)s",
                        f"{ns_cmd_prefix} ethtool -m %(dev)s"
                    ], devices=_devs['ethernet'], priority=50, subdir=_subdir)

        self.add_command_tags()


class RedHatNetworking(Networking, RedHatPlugin):
    trace_host = "rhn.redhat.com"

    def setup(self):
        # Handle change from -T to -W in Red Hat netstat 2.0 and greater.
        try:
            netstat_pkg = self.policy.package_manager.pkg_by_name('net-tools')
            # major version
            if int(netstat_pkg['version'][0]) < 2:
                self.ns_wide = "-T"
        except Exception:  # pylint: disable=broad-except
            # default to upstream option
            pass

        super().setup()

    def postproc(self):

        self.do_path_regex_sub(
            "/etc/nmstate",
            r"(\s+(mka-cak|private-key-password|psk|password):).*",
            r"\1 ******"
        )


class UbuntuNetworking(Networking, UbuntuPlugin, DebianPlugin):
    trace_host = "archive.ubuntu.com"

    def setup(self):

        common_ss_kmods = ['af_packet_diag', 'inet_diag', 'mptcp_diag',
                           'netlink_diag', 'raw_diag', 'tcp_diag', 'udp_diag',
                           'unix_diag']

        if (isinstance(self.policy, UbuntuPolicy) and
                self.policy.dist_version() >= 22.04):
            self.ss_kmods = common_ss_kmods + ['xsk_diag']
        elif (isinstance(self.policy, DebianPolicy) and
                self.policy.dist_version() >= 13):
            self.ss_kmods = common_ss_kmods + ['vsock_diag']

        super().setup()

        self.add_copy_spec([
            "/etc/netplan/*.yaml",
            "/etc/network/interfaces",
            "/etc/network/interfaces.d",
            "/etc/resolv.conf",
            "/etc/resolvconf",
            "/lib/netplan/*.yaml",
            "/run/netplan/*.yaml",
            "/run/systemd/network"
        ])

        # Netplan only consumes files with the `.yaml` extension (LP#1815734),
        # so give visibility on other files that might be present.
        self.add_dir_listing([
            "/etc/netplan",
            "/lib/netplan",
            "/run/netplan",
        ])

    def postproc(self):

        self.do_path_regex_sub(
            "/etc/netplan",
            r"(\s+password:).*",
            r"\1 ******"
        )


# vim: set et ts=4 sw=4 :



# Copyright (C) 2020 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re

from shlex import quote
from sos.utilities import sos_get_command_output, is_executable


class ContainerRuntime():
    """Encapsulates a container runtime that provides the ability to plugins to
    check runtime status, check for the presence of specific containers, and
    to format commands to run in those containers

    :param policy: The loaded policy for the system
    :type policy: ``Policy()``

    :cvar name: The name of the container runtime, e.g. 'podman'
    :vartype name: ``str``

    :cvar containers: A list of containers known to the runtime
    :vartype containers: ``list``

    :cvar images: A list of images known to the runtime
    :vartype images: ``list``

    :cvar binary: The binary command to run for the runtime, must exit within
                  $PATH
    :vartype binary: ``str``
    """

    name = 'Undefined'
    containers = []
    images = []
    volumes = []
    binary = ''
    active = False

    def __init__(self, policy=None):
        self.policy = policy
        self.run_cmd = f"{self.binary} exec "

    def load_container_info(self):
        """If this runtime is found to be active, attempt to load information
        on the objects existing in the runtime.
        """
        self.containers = self.get_containers()
        self.images = self.get_images()
        self.volumes = self.get_volumes()

    def check_is_active(self):
        """Check to see if the container runtime is both present AND active.

        Active in this sense means that the runtime can be used to glean
        information about the runtime itself and containers that are running.

        :returns: ``True`` if the runtime is active, else ``False``
        :rtype: ``bool``
        """
        if is_executable(self.binary, self.policy.sysroot):
            self.active = True
            return True
        return False

    def check_can_copy(self):
        """Check if the runtime supports copying files out of containers and
        onto the host filesystem
        """
        return True

    def get_containers(self, get_all=False):
        """Get a list of containers present on the system.

        :param get_all: If set, include stopped containers as well
        :type get_all: ``bool``
        """
        containers = []
        _cmd = f"{self.binary} ps {'-a' if get_all else ''}"
        if self.active:
            out = sos_get_command_output(_cmd, chroot=self.policy.sysroot)
            if out['status'] == 0:
                for ent in out['output'].splitlines()[1:]:
                    ent = ent.split()
                    # takes the form (container_id, container_name)
                    containers.append((ent[0], ent[-1]))
        return containers

    def get_container_by_name(self, name):
        """Get the container ID for the container matching the provided
        name

        :param name: The name of the container, note this can be a regex
        :type name: ``str``

        :returns: The id of the first container to match `name`, else ``None``
        :rtype: ``str``
        """
        if not self.active or name is None:
            return None
        for c in self.containers:
            if re.match(name, c[1]):
                return c[0]
        return None

    def get_images(self):
        """Get a list of images present on the system

        :returns: A list of 2-tuples containing (image_name, image_id)
        :rtype: ``list``
        """
        images = []
        fmt = '{{lower .Repository}}:{{lower .Tag}} {{lower .ID}}'
        if self.active:
            out = sos_get_command_output(
                f"{self.binary} images --format '{fmt}'",
                chroot=self.policy.sysroot
            )
            if out['status'] == 0:
                for ent in out['output'].splitlines():
                    ent = ent.split()
                    # takes the form (image_name, image_id)
                    images.append((ent[0], ent[1]))
        return images

    def get_volumes(self):
        """Get a list of container volumes present on the system

        :returns: A list of volume IDs on the system
        :rtype: ``list``
        """
        vols = []
        if self.active:
            out = sos_get_command_output(
                f"{self.binary} volume ls",
                chroot=self.policy.sysroot
            )
            if out['status'] == 0:
                for ent in out['output'].splitlines()[1:]:
                    ent = ent.split()
                    vols.append(ent[-1])
        return vols

    def container_exists(self, container):
        """Check if a given container ID or name exists on the system from the
        perspective of the container runtime.

        Note that this will only check _running_ containers

        :param container:       The name or ID of the container
        :type container:        ``str``

        :returns:               True if the container exists, else False
        :rtype:                 ``bool``
        """
        for _contup in self.containers:
            if container in _contup:
                return True
        return False

    def fmt_container_cmd(self, container, cmd, quotecmd):
        """Format a command to run inside a container using the runtime

        :param container: The name or ID of the container in which to run
        :type container: ``str``

        :param cmd: The command to run inside `container`
        :type cmd: ``str``

        :param quotecmd: Whether the cmd should be quoted.
        :type quotecmd: ``bool``

        :returns: Formatted string to run `cmd` inside `container`
        :rtype: ``str``
        """
        if quotecmd:
            quoted_cmd = quote(cmd)
        else:
            quoted_cmd = cmd
        return f"{self.run_cmd} {container} {quoted_cmd}"

    def fmt_registry_credentials(self, username, password):
        """Format a string to pass to the 'run' command of the runtime to
        enable authorization for pulling the image during `sos collect`, if
        needed using username and optional password creds

        :param username:    The name of the registry user
        :type username:     ``str``

        :param password:    The password of the registry user
        :type password:     ``str`` or ``None``

        :returns:  The string to use to enable a run command to pull the image
        :rtype:    ``str``
        """
        return f"--creds={username}{':' + password if password else ''}"

    def fmt_registry_authfile(self, authfile):
        """Format a string to pass to the 'run' command of the runtime to
        enable authorization for pulling the image during `sos collect`, if
        needed using an authfile.
        """
        if authfile:
            return f"--authfile {authfile}"
        return ''

    def get_logs_command(self, container):
        """Get the command string used to dump container logs from the
        runtime

        :param container: The name or ID of the container to get logs for
        :type container: ``str``

        :returns: Formatted runtime command to get logs from `container`
        :type: ``str``
        """
        return f"{self.binary} logs -t {container}"

    def get_copy_command(self, container, path, dest, sizelimit=None):
        """Generate the command string used to copy a file out of a container
        by way of the runtime.

        :param container:   The name or ID of the container
        :type container:    ``str``

        :param path:        The path to copy from the container. Note that at
                            this time, no supported runtime supports globbing
        :type path:         ``str``

        :param dest:        The destination on the *host* filesystem to write
                            the file to
        :type dest:         ``str``

        :param sizelimit:   Limit the collection to the last X bytes of the
                            file at PATH
        :type sizelimit:    ``int``

        :returns:   Formatted runtime command to copy a file from a container
        :rtype:     ``str``
        """
        if sizelimit:
            return f"{self.run_cmd} {container} tail -c {sizelimit} {path}"
        return f"{self.binary} cp {container}:{path} {dest}"

# vim: set et ts=4 sw=4 :



# Copyright Red Hat 2022, Trevor Benson <trevor.benson@gmail.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json
from shlex import quote
from sos.collector.clusters import Cluster


class saltstack(Cluster):
    """
    The saltstack cluster profile is intended to be used on saltstack
    clusters (Salt Project).
    """

    cluster_name = "Saltstack"
    packages = ("salt-master",)
    sos_plugins = ["saltmaster"]
    strict_node_list = True
    option_list = [
        ("compound", "", "Filter node list to those matching compound"),
        ("glob", "", "Filter node list to those matching glob pattern"),
        ("grain", "", "Filter node list to those with matching grain"),
        ("minion_id_unresolvable", False, "Returns the FQDN grain of each"
         " minion in the node list when the minion ID is not a hostname."),
        ("nodegroup", "", "Filter node list to those matching nodegroup"),
        ("pillar", "", "Filter node list to those with matching pillar"),
        ("subnet", "", "Filter node list to those in subnet"),
    ]
    targeted = False

    node_cmd = "salt-run --out=pprint manage.status"

    def _parse_manage_status(self, output: str) -> list:
        nodes = []
        salt_json_output = json.loads(output.replace("'", '"'))
        for _, value in salt_json_output.items():
            nodes.extend(value)
        return nodes

    def _get_hostnames_from_grain(self, manage_status: dict) -> list:
        hostnames = []
        for status, minions in manage_status.items():
            if status == "down":
                self.log_warn(f"Node(s) {minions} are status down.")
                hostnames.extend(minions)
            else:
                for minion in minions:
                    node_cmd = (
                        f"salt --out=newline_values_only {minion} "
                        f"grains.get fqdn"
                    )
                    hostnames.append(
                        self.exec_primary_cmd(node_cmd)["output"].strip()
                    )
        return hostnames

    def _get_nodes(self) -> list:
        res = self.exec_primary_cmd(self.node_cmd)
        if res["status"] != 0:
            raise Exception("Node enumeration did not return usable output")
        if self.get_option("minion_id_unresolvable"):
            status = json.loads(res["output"].replace("'", '"'))
            return self._get_hostnames_from_grain(status)
        return self._parse_manage_status(res["output"])

    def get_nodes(self):
        # Default to all online nodes
        for option in self.option_list:
            if option[0] != "minion_id_unresolvable":
                opt = self.get_option(option[0])
                if opt:
                    self.node_cmd += f" tgt={quote(opt)} tgt_type={option[0]}"
                    break
        return self._get_nodes()


# vim: set et ts=4 sw=4 :



# Copyright (C) 2015 Red Hat, Inc.,Poornima M. Kshirsagar <pkshiras@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenStackSahara(Plugin):

    short_desc = 'OpenStack Sahara'
    plugin_name = 'openstack_sahara'
    profiles = ('openstack', 'openstack_controller')
    var_puppet_gen = "/var/lib/config-data/puppet-generated/sahara"

    def setup(self):
        self.add_copy_spec([
            "/etc/sahara/",
            self.var_puppet_gen + "/etc/sahara/"
        ])
        self.add_journal(units="openstack-sahara-all")
        self.add_journal(units="openstack-sahara-api")
        self.add_journal(units="openstack-sahara-engine")

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/sahara/",
            ])
        else:
            self.add_copy_spec([
                "/var/log/sahara/*.log",
            ])

    def apply_regex_sub(self, regexp, subst):
        """ Apply regex substitution """
        self.do_path_regex_sub("/etc/sahara/*", regexp, subst)
        self.do_path_regex_sub(
            self.var_puppet_gen + "/etc/sahara/*",
            regexp, subst
        )

    def postproc(self):
        protect_keys = [
            "admin_password", "memcache_secret_key", "password",
            "qpid_password", "rabbit_password", "ssl_key_password",
            "xenapi_connection_password", "transport_url"
        ]
        connection_keys = ["connection"]

        join_con_keys = "|".join(connection_keys)

        self.apply_regex_sub(
            fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)",
            r"\1*********"
        )
        self.apply_regex_sub(
            fr"(^\s*({join_con_keys})\s*=\s*(.*)://(\w*):)(.*)(@(.*))",
            r"\1*********\6"
        )


class DebianSahara(OpenStackSahara, DebianPlugin, UbuntuPlugin):

    short_desc = 'OpenStack Sahara information for Debian based distributions'
    packages = (
        'sahara-api',
        'sahara-common',
        'sahara-engine',
        'python-sahara',
        'python3-sahara',
    )


class RedHatSahara(OpenStackSahara, RedHatPlugin):

    short_desc = 'OpenStack Sahara information for Red Hat distributions'
    packages = ('openstack-selinux',)

    def setup(self):
        super().setup()
        self.add_copy_spec("/etc/sudoers.d/sahara*")


# vim: et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, UbuntuPlugin


# See the docs: https://documentation.ubuntu.com/authd/
class Authd(Plugin, UbuntuPlugin):

    short_desc = 'Authd daemon & broker information'

    plugin_name = 'authd'

    apt_packages = (
        'authd',
    )

    snap_packages = (
        'authd-msentraid',
        'authd-google',
    )

    packages = apt_packages + snap_packages

    services = (
        'authd',
        'snap.authd-msentraid.authd-msentraid',
        'snap.authd-google.authd-google',
    )

    def setup(self):
        self.add_dir_listing([
            "/etc/authd/brokers.d",
        ])

        self.add_copy_spec([
            "/etc/authd/brokers.d/msentraid.conf",
            "/etc/authd/brokers.d/google.conf",
            "/var/snap/authd-google/current/broker.conf",
            "/var/snap/authd-google/current/broker.conf.d/*",
            "/var/snap/authd-msentraid/current/broker.conf",
            "/var/snap/authd-msentraid/current/broker.conf.d/*",
        ])

        self.add_cmd_output([
            f"apt-cache policy {' '.join(self.apt_packages)}",
            f"snap list --all {' '.join(self.snap_packages)}",
            "/usr/libexec/authd version",
        ])

    def postproc(self):
        # Entra uses hex encoded IDs/secrets so just filter all hex data (with
        # `-`) to be safe. These can be generated with uuidgen:
        # $ uuidgen
        # dd591ced-483e-4c47-beaf-ff46f68aab0a
        self.do_path_regex_sub(
            r".*",
            r"[a-fA-F0-9-]{18,}",
            r"******",
        )

        # Google's encoding is less clear, so we'll just filter out the values
        # of the config fields (client_id and client_secret):
        # client_secret = some.base64.stuff.with.domain
        self.do_path_regex_sub(
            r".*",
            r"(.*_(id|secret)\s*=\s*)(.*)",
            r"\1******",
        )



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.cleaner.mappings import SoSMap


class SoSUsernameMap(SoSMap):
    """Mapping to store usernames matched from ``lastlog`` output.

    Usernames are obfuscated as ``obfuscateduserX`` where ``X`` is a counter
    that gets incremented for every new username found.

    Note that this specifically obfuscates user_names_ and not UIDs.
    """

    ignore_short_items = True
    match_full_words_only = True
    name_count = 0

    def sanitize_item(self, item):
        """Obfuscate a new username not currently found in the map
        """
        ob_name = f"obfuscateduser{self.name_count}"
        self.name_count += 1
        if ob_name in self.dataset.values():
            return self.sanitize_item(item.lower())
        return ob_name



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json
import re

from sos_tests import StageTwoReportTest


class FullCleanTest(StageTwoReportTest):
    """Run an unrestricted report execution through sos clean, ensuring that
    our obfuscation is reliable across arbitrary plugin sets and not just the
    'main' plugins that tend to collect data needing obfuscation

    :avocado: tags=stagetwo
    """

    sos_cmd = '--clean'
    sos_timeout = 600
    # replace with an empty placeholder, make sure that this test case is not
    # influenced by previous clean runs
    files = [('default_mapping', '/etc/sos/cleaner/default_mapping')]
    packages = {
        'rhel': ['python3-systemd'],
        'ubuntu': ['python3-systemd']
    }

    def pre_sos_setup(self):
        # ensure that case-insensitive matching of FQDNs and shortnames work
        from systemd import journal
        from socket import gethostname
        host = gethostname()
        short = host.split('.')[0]
        sosfd = journal.stream('sos-testing')
        sosfd.write(
            f"This is a test line from sos clean testing. The hostname "
            f"{host.lower()} should not appear, nor should {host.upper()} "
            f"in an obfuscated archive. The shortnames of {short.lower()} "
            f"and {short.upper()} should also not appear."
        )

    def test_private_map_was_generated(self):
        self.assertOutputContains(
            'A mapping of obfuscated elements is available at'
        )
        map_file = re.findall(
            '/.*sosreport-.*-private_map',
            self.cmd_output.stdout
        )[-1]
        self.assertFileExists(map_file)

    def test_tarball_named_obfuscated(self):
        self.assertTrue('obfuscated' in self.archive)

    def test_archive_type_correct(self):
        self.assertSosLogContains('Loaded .* as type sos report directory')

    def test_hostname_not_in_any_file(self):
        host = self.sysinfo['pre']['networking']['hostname']
        short = host.split('.')[0]
        # much faster to just use grep here
        host = rf'(?<![a-z0-9]){re.escape(host)}(?=\b|_|-)'
        short = rf'(?<![a-z0-9]){re.escape(short)}(?=\b|_|-)'
        content = self.grep_for_content(host, regexp=True) + \
            self.grep_for_content(short, regexp=True)
        if not content:
            assert True
        else:
            self.fail("Hostname appears in files: %s"
                      % "\n".join(f for f in content))

    def test_no_empty_obfuscations(self):
        # get the private map file name
        map_file = re.findall(
            '/.*sosreport-.*-private_map',
            self.cmd_output.stdout
        )[-1]
        with open(map_file, 'r', encoding='utf-8') as mf:
            map_json = json.load(mf)
        for mapping in map_json:
            for key, val in map_json[mapping].items():
                assert key, f"Empty key found in {mapping}"
                assert val, f"{mapping} mapping for '{key}' empty"

    def test_ip_not_in_any_file(self):
        ip = self.sysinfo['pre']['networking']['ip_addr']
        content = self.grep_for_content(ip)
        if not content:
            assert True
        else:
            new_content = "\n".join(f for f in content)
            self.fail(f'IP appears in files: {new_content}')



# Copyright (c) 2024 Mike Silmser <msilmser@redhat.com>
# Copyright (c) 2024 Lucas Benedito <lbenedit@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class AAPAutomationHub(Plugin, RedHatPlugin):
    short_desc = 'AAP Automation Hub plugin'
    plugin_name = 'aap_hub'
    profiles = ('sysmgmt', 'ansible',)
    packages = ('automation-hub',)

    def setup(self):
        self.add_copy_spec([
            "/etc/ansible-automation-platform/",
            "/var/log/ansible-automation-platform/hub/worker.log*",
            "/var/log/ansible-automation-platform/hub/pulpcore-api.log*",
            "/var/log/ansible-automation-platform/hub/pulpcore-content.log*",
            "/var/log/nginx/automationhub.access.log*",
            "/var/log/nginx/automationhub.error.log*",

        ])

        self.add_dir_listing([
            "/etc/ansible-automation-platform/",
            "/var/log/ansible-automation-platform/",
        ], recursive=True)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2024 Alejandro Santoyo <alejandro.santoyo@canonical.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class InfinidatStorage(Plugin, IndependentPlugin):

    short_desc = 'Infinidat Storage plugin'
    plugin_name = 'infinidat'
    profiles = ('storage',)
    packages = ('host-power-tools',)

    def setup(self):
        # Get infinidat logs
        if not self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/infinihost.latest*.log",
                "/var/log/infinihost.usage*.log",
            ])
        else:
            self.add_copy_spec([
                "/var/log/infinihost*.log",
                "/var/log/buildout.*.log",
            ])

        # Get info from the infinidat boxes, etc.
        self.add_cmd_output([
            "infinihost volume list",
            "infinihost connectivity list",
            "infinihost system list",
            "infinihost pool list",
            "infinihost snapshot list",
            "infinihost --version"
        ])

# vim: set et ts=4 sw=4 :



# Copyright Red Hat 2020, Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


class InvalidPasswordException(Exception):
    """Raised when the provided password is rejected by the remote host"""

    def __init__(self):
        message = 'Invalid password provided'
        super().__init__(message)


class TimeoutPasswordAuthException(Exception):
    """Raised when a timeout is hit waiting for an auth reply using a password
    """

    def __init__(self):
        message = 'Timeout hit while waiting for password validation'
        super().__init__(message)


class PasswordRequestException(Exception):
    """Raised when the remote host requests a password that was not anticipated
    """

    def __init__(self):
        message = 'Host requested password, but none provided'
        super().__init__(message)


class AuthPermissionDeniedException(Exception):
    """Raised when authentication attempts return a permission error"""

    def __init__(self):
        message = 'Permission denied while trying to authenticate'
        super().__init__(message)


class ConnectionException(Exception):
    """Raised when an attempt to connect fails"""

    def __init__(self, address='', port=''):
        message = (f"Could not connect to host {address} on specified port "
                   f"{port}")
        super().__init__(message)


class CommandTimeoutException(Exception):
    """Raised when a timeout expires"""

    def __init__(self, command=None):
        message = 'Timeout expired'
        if command:
            message += f" executing {command}"
        super().__init__(message)


class ConnectionTimeoutException(Exception):
    """Raised when a timeout expires while trying to connect to the host"""

    def __init__(self):
        message = 'Timeout expires while trying to connect'
        super().__init__(message)


class ControlSocketMissingException(Exception):
    """Raised when the SSH control socket is missing"""

    def __init__(self, path=''):
        message = f"SSH control socket {path} does not exist"
        super().__init__(message)


class ControlPersistUnsupportedException(Exception):
    """Raised when SSH ControlPersist is unsupported locally"""

    def __init__(self):
        message = 'ControlPersist unsupported by local SSH installation'
        super().__init__(message)


class UnsupportedHostException(Exception):
    """Raised when the host type is unsupported or undetermined"""

    def __init__(self):
        message = 'Host did not match any supported distributions'
        super().__init__(message)


class InvalidTransportException(Exception):
    """Raised when a transport is requested but it does not exist or is
    not supported locally"""

    def __init__(self, transport=None):
        message = ("Connection failed: unknown or unsupported transport "
                   f"{transport if transport else ''}")
        super().__init__(message)


class SaltStackMasterUnsupportedException(Exception):
    """Raised when SaltStack Master is unsupported locally"""

    def __init__(self):
        message = 'Master unsupported by local SaltStack installation'
        super().__init__(message)


class JujuNotInstalledException(Exception):
    """Raised when juju is not installed locally"""

    def __init__(self):
        message = (
            'Juju is not installed, '
            'please ensure you have installed juju.'
        )
        super().__init__(message)


__all__ = [
    'AuthPermissionDeniedException',
    'CommandTimeoutException',
    'ConnectionException',
    'ConnectionTimeoutException',
    'ControlPersistUnsupportedException',
    'ControlSocketMissingException',
    'InvalidPasswordException',
    'PasswordRequestException',
    'SaltStackMasterUnsupportedException',
    'TimeoutPasswordAuthException',
    'UnsupportedHostException',
    'InvalidTransportException',
    'JujuNotInstalledException'
]



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
from sos.report.plugins import Plugin, UbuntuPlugin


class Landscape(Plugin, UbuntuPlugin):

    short_desc = 'Ubuntu Landscape client'

    plugin_name = 'landscape'
    profiles = ('sysmgmt',)
    files = ('/etc/landscape/client.conf', '/etc/landscape/service.conf')
    packages = ('landscape-client', 'landscape-server')

    def setup(self):

        vars_all = [p in os.environ for p in [
                        'LANDSCAPE_API_KEY',
                        'LANDSCAPE_API_SECRET',
                        'LANDSCAPE_API_URI',
                    ]]

        if not all(vars_all):
            self.soslog.warning("Not all environment variables set. "
                                "Source the environment file for the user "
                                "intended to connect to the Landscape "
                                "environment so that the landscape-api "
                                "commands can be used.")
        else:
            self.add_cmd_output([
                "landscape-api get-distributions",
                "landscape-api get-apt-sources",
                "landscape-api get-repository-profiles",
                "landscape-api get activites --limit 100",
            ])
            self.add_cmd_output([
                "landscape-api --json get-distributions",
                "landscape-api --json get-apt-sources",
                "landscape-api --json get-repository-profiles",
                "landscape-api --json get activites --limit 100",
            ])

        self.add_copy_spec([
            "/etc/default/landscape-client",
            "/etc/default/landscape-server",
            "/etc/landscape/client.conf",
            "/etc/landscape/service.conf",
            "/etc/landscape/service.conf.old",
            "/var/lib/landscape/landscape-oops/*/OOPS-*"
        ])

        if not self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/landscape/*.log",
                "/var/log/landscape-server/*.log",
            ])
        else:
            self.add_copy_spec([
                "/var/log/landscape",
                "/var/log/landscape-server"
            ])

        self.add_cmd_output([
            "gpg --verify /etc/landscape/license.txt",
            "head -n 5 /etc/landscape/license.txt",
            "lsctl status"
        ])

    def postproc(self):
        self.do_file_sub(
            "/etc/landscape/client.conf",
            r"registration_password(.*)",
            r"registration_password[********]"
        )
        keys = [
            "password",
            "store_password",
            "secret-token",
            "oidc-client-secret",
            "oidc-client-id",
        ]
        self.do_path_regex_sub(
            "/etc/landscape/service.conf*",
            fr"({'|'.join(keys)}) = (.*)",
            r"\1 = [********]"
        )

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import json
from http.client import HTTPResponse
from typing import Any
from urllib import request
from urllib.error import URLError

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class GCP(Plugin, IndependentPlugin):

    short_desc = 'Google Cloud Platform'
    plugin_name = 'gcp'
    profiles = ('virt',)

    option_list = [
        PluginOpt('keep-pii', default=False,
                  desc="Stop the plugin from removing PIIs like project name "
                       "or organization ID from the metadata retrieved from "
                       "Metadata server.")
    ]

    METADATA_ROOT = "http://metadata.google.internal/computeMetadata/v1/"
    METADATA_QUERY = "http://metadata.google.internal/computeMetadata/v1/" \
                     "?recursive=true"
    REDACTED = "[--REDACTED--]"
    metadata = None

    # A line we will be looking for in the dmesg output. If it's there,
    # that means we're running on a Google Cloud Compute instance.
    GOOGLE_DMI = "DMI: Google Google Compute Engine/Google " \
                 "Compute Engine, BIOS Google"

    def check_enabled(self):
        """
        Checks if this plugin should be executed at all. In this case, it
        will check the `dmesg` command output to see if the system is
        running on a Google Cloud Compute instance.
        """
        dmesg = self.exec_cmd("dmesg")
        if dmesg['status'] != 0:
            return False
        return self.GOOGLE_DMI in dmesg['output']

    def setup(self):
        """
        Collect the following info:
         * Metadata from the Metadata server
         * `gcloud auth list` output
         * Any google services output from journal
        """

        # Capture gcloud auth list
        self.add_cmd_output("gcloud auth list", tags=['gcp'])

        # Add journal entries
        self.add_journal(units="google*", tags=['gcp'])

    def collect(self):
        # Get and store Metadata
        with self.collection_file('metadata.json', tags=['gcp']) as mfile:
            try:
                self.metadata = self.get_metadata()
                self.scrub_metadata()
                mfile.write(json.dumps(self.metadata, indent=4))
            except RuntimeError as err:
                mfile.write(str(err))

    def get_metadata(self) -> dict:
        """
        Retrieves metadata from the Metadata Server and transforms it into a
        dictionary object.
        """
        response = self._query_address(self.METADATA_QUERY)
        response_body = response.read().decode()
        return json.loads(response_body)

    @staticmethod
    def _query_address(url: str) -> HTTPResponse:
        """
        Query the given url address with headers required by Google Metadata
        Server.
        """
        try:
            req = request.Request(url, headers={'Metadata-Flavor': 'Google'})
            with request.urlopen(req) as response:
                if response.code != 200:
                    raise RuntimeError(
                        f"Failed to communicate with Metadata Server "
                        f"(code: {response.code}): " +
                        response.read().decode())
                return response
        except URLError as err:
            raise RuntimeError(
                "Failed to communicate with Metadata Server: " + str(err)) \
                from err

    def scrub_metadata(self):
        """
        Remove all PII information from metadata, unless a keep-pii option
        is specified.

        Note: PII information collected by this plugin, like
        project number, account names etc. might be required by Google
        Cloud Support for faster issue resolution.
        """
        if self.get_option('keep-pii'):
            return

        project_id = self.metadata['project']['projectId']
        project_number_int = self.metadata['project']['numericProjectId']
        project_number = str(project_number_int)

        def scrub(data: Any) -> Any:
            if isinstance(data, dict):
                if 'token' in data:
                    # Data returned for recursive query shouldn't contain
                    # tokens, but you can't be too careful.
                    data['token'] = self.REDACTED
                return {scrub(k): scrub(v) for k, v in data.items()}
            if isinstance(data, list):
                return [scrub(value) for value in data]
            if isinstance(data, str):
                return data.replace(project_number, self.REDACTED)\
                           .replace(project_id, self.REDACTED)
            if isinstance(data, int):
                return self.REDACTED if data == project_number_int else data
            return data

        self.metadata = scrub(self.metadata)

        self.safe_redact_key(self.metadata['project']['attributes'],
                             'ssh-keys')
        self.safe_redact_key(self.metadata['project']['attributes'],
                             'sshKeys')

    @classmethod
    def safe_redact_key(cls, dict_obj: dict, key: str):
        """ Redact keys """
        if key in dict_obj:
            dict_obj[key] = cls.REDACTED



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Anaconda(Plugin, RedHatPlugin):

    short_desc = 'Anaconda installer'

    plugin_name = 'anaconda'
    profiles = ('system',)
    packages = ('anaconda',)

    files = (
        '/var/log/anaconda.log',
        '/var/log/anaconda'
    )

    def setup(self):

        self.copypaths = [
            "/root/anaconda-ks.cfg"
        ]

        if self.path_isdir('/var/log/anaconda'):
            # new anaconda
            self.copypaths.append('/var/log/anaconda')
        else:
            self.copypaths = self.copypaths + [
                "/var/log/anaconda.*",
                "/root/install.log",
                "/root/install.log.syslog"
            ]

        self.add_copy_spec(self.copypaths)

    def postproc(self):
        self.do_file_sub(
            "/root/anaconda-ks.cfg",
            r"(\s*rootpw\s*).*",
            r"\1********"
        )
        self.do_file_sub(
            "/root/anaconda-ks.cfg",
            r"(user.*--password=*\s*)\s*(\S*)",
            r"\1********"
        )
        self.do_paths_http_sub(self.copypaths)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos.report.plugins import Plugin, IndependentPlugin, SoSPredicate


class DeviceMapper(Plugin, IndependentPlugin):

    short_desc = 'device-mapper framework'

    plugin_name = 'devicemapper'
    profiles = ('storage',)
    packages = ('device-mapper',)
    kernel_mods = ('dm_mod', )
    files = ('/dev/mapper',)

    def setup(self):
        self.add_cmd_output([
            "dmsetup info -c",
            "dmsetup table",
            "dmsetup status",
            "dmsetup ls --tree",
            "dmsetup udevcookies",
            "dmstats list",
            "dmstats print --allregions"
        ], pred=SoSPredicate(self, kmods=['dm_mod']))

        self.add_cmd_tags({
            "dmsetup info -c": "dmsetup_info",
            "dmsetup status": "dmsetup_status"
        })

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import json
from socket import gethostname
from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin


class CephCommon(Plugin, RedHatPlugin, UbuntuPlugin):

    short_desc = 'CEPH common'

    plugin_name = 'ceph_common'
    profiles = ('storage', 'virt', 'container', 'ceph')

    containers = ('ceph-(.*-)?(mon|rgw|osd).*',)
    ceph_hostname = gethostname()

    packages = (
        'ceph',
        'ceph-mds',
        'ceph-common',
        'libcephfs1',
        'ceph-fs-common',
        'calamari-server',
    )

    services = (
        'ceph-nfs@pacemaker',
        f'ceph-mds@{ceph_hostname}',
        f'ceph-mon@{ceph_hostname}',
        f'ceph-mgr@{ceph_hostname}',
        'ceph-radosgw@*',
        'ceph-osd@*'
    )

    # This check will enable the plugin regardless of being
    # containerized or not
    files = ('/etc/ceph/ceph.conf',
             '/var/snap/microceph/*',)

    def setup(self):
        all_logs = self.get_option("all_logs")

        microceph_pkg = self.policy.package_manager.pkg_by_name('microceph')
        if not microceph_pkg:
            self.add_file_tags({
                '.*/ceph.conf': 'ceph_conf',
                '/var/log/ceph(.*)?/ceph.log.*': 'ceph_log',
            })

            if not all_logs:
                self.add_copy_spec([
                    "/var/log/calamari/*.log",
                    "/var/log/ceph/**/ceph.log",
                    "/var/log/ceph/cephadm.log",
                ])
            else:
                self.add_copy_spec([
                    "/var/log/calamari",
                    "/var/log/ceph/**/ceph.log*",
                    "/var/log/ceph/cephadm.log*",
                ])

            self.add_copy_spec([
                "/var/log/ceph/**/ceph.audit.log*",
                "/etc/ceph/",
                "/etc/calamari/",
                "/var/lib/ceph/tmp/",
            ])

            self.add_forbidden_path([
                "/etc/ceph/*keyring*",
                "/var/lib/ceph/*keyring*",
                "/var/lib/ceph/*/*keyring*",
                "/var/lib/ceph/*/*/*keyring*",
                "/var/lib/ceph/osd",
                "/var/lib/ceph/mon",
                # Excludes temporary ceph-osd mount location like
                # /var/lib/ceph/tmp/mnt.XXXX from sos collection.
                "/var/lib/ceph/tmp/*mnt*",
                "/etc/ceph/*bindpass*"
            ])
        else:
            if not all_logs:
                self.add_copy_spec([
                    "/var/snap/microceph/common/logs/ceph.log",
                    "/var/snap/microceph/common/logs/ceph.audit.log",
                ])
            else:
                self.add_copy_spec([
                    "/var/snap/microceph/common/logs/ceph.log*",
                    "/var/snap/microceph/common/logs/ceph.audit.log*",
                ])

            self.add_cmd_output("snap info microceph", subdir="microceph")

            cmds = [
                'client config list',
                'cluster config list',
                'cluster list',
                'disk list',
                'log get-level',
                'status',
                'pool list',
                'remote list',
                'replication list rbd',
            ]

            self.add_cmd_output([f"microceph {cmd}" for cmd in cmds],
                                subdir='microceph')

            dqlite_crt = "/var/snap/microceph/common/state/cluster.crt"
            self.add_cmd_output(
                f"openssl x509 -in {dqlite_crt} -noout -dates",
                subdir="microceph"
            )

            db_path = "/var/snap/microceph/common/state/database"

            # Check for inconsistent dqlite db intervals
            self.add_dir_listing(
                db_path,
                suggest_filename="ls_microceph_dqlite_dir",
                subdir="microceph",
            )

            self.add_copy_spec([
                    f"{db_path}/info.yaml",
                    f"{db_path}/cluster.yaml",
                    f"{db_path}/../daemon.yaml",
            ])

            queries = [
                {
                    "query": (
                        "SELECT * FROM sqlite_master WHERE type=\"table\";"
                    ),
                    "suggested_file_suffix": "schema",
                },
                {
                    "query": (
                        "SELECT * FROM config WHERE NOT ( "
                        "key LIKE \"%keyring%\" OR "
                        "key LIKE \"%ca_cert%\" OR "
                        "key LIKE \"%ca_key%\" );"
                    ),
                    "suggested_file_suffix": "config",
                },
                {
                    "query": "SELECT * FROM services;",
                    "suggested_file_suffix": "services",
                },
                {
                    "query": (
                        "SELECT id, name, expiry_date "
                        "FROM core_token_records;"
                    ),
                    "suggested_file_suffix": "token_records",
                },
                {
                    "query": (
                        "SELECT id, name, address, schema_internal, "
                        "schema_external, heartbeat, role, api_extensions "
                        "FROM core_cluster_members;"
                    ),
                    "suggested_file_suffix": "core_cluster_members",
                },
                {
                    "query": "SELECT * FROM disks;",
                    "suggested_file_suffix": "disks",
                },
                {
                    "query": "SELECT * FROM client_config;",
                    "suggested_file_suffix": "client_config",
                },
                {
                    "query": "SELECT * FROM remote;",
                    "suggested_file_suffix": "remote",
                },
            ]

            for query_entry in queries:
                query = json.dumps(query_entry.get("query"))
                file_suffix = query_entry.get("suggested_file_suffix")
                self.add_cmd_output(
                    f"microceph cluster sql {query}",
                    suggest_filename=f"microceph_cluster_sql_{file_suffix}",
                    subdir="microceph"
                )

        self.add_cmd_output([
            "ceph -v",
        ])

    def postproc(self):
        protect_keys = [
            "rgw keystone admin password",
        ]
        regex = fr"(^({'|'.join(protect_keys)})\s*=\s*)(.*)"
        self.do_path_regex_sub("/etc/ceph/ceph.conf", regex, r"\1*********")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class LibvirtClient(Plugin, IndependentPlugin):

    short_desc = 'client for libvirt virtualization API'

    plugin_name = 'virsh'
    profiles = ('system', 'virt')

    packages = ('libvirt-client', 'libvirt-clients',)

    def setup(self):
        # virt-manager logs
        self.add_copy_spec([
            "/root/.cache/virt-manager/*.log",
            "/root/.virt-manager/*.log"
        ])

        cmd = 'virsh -r'

        # get host information
        subcmds = [
            'domcapabilities',
            'capabilities',
            'nodeinfo',
            'freecell --all',
            'node-memory-tune',
            'version',
            'pool-capabilities',
            'nodecpumap',
            'maxvcpus kvm',
            'sysinfo',
            'nodedev-list --tree',
        ]

        for subcmd in subcmds:
            self.add_cmd_output(f'{cmd} {subcmd}', foreground=True)

        self.add_cmd_output(f"{cmd} list --all",
                            tags="virsh_list_all", foreground=True)

        vms = self.exec_cmd(f"{cmd} list --all --name --state-running "
                            "--state-paused", foreground=True)
        if vms['status'] == 0:
            for vm in vms['output'].splitlines():
                pid = self.exec_cmd(f"pgrep -f {vm}")
                if pid['status'] == 0:
                    # We're grabbing the first item, as the second one is the
                    # item from the process list that is just pgrep
                    p = pid['output'].splitlines()[0]
                    self.add_cmd_output(
                        f"taskset -ac -p {p}",
                        suggest_filename=f"taskset_{vm}")

        # get network, pool and nwfilter elements
        for k in ['net', 'nwfilter', 'pool']:
            k_list = self.collect_cmd_output(f'{cmd} {k}-list %s' % ('--all'
                                             if k in ['net', 'pool'] else ''),
                                             foreground=True)
            if k_list['status'] == 0:
                k_lines = k_list['output'].splitlines()
                # the 'Name' column position changes between virsh cmds
                # catch the rare exceptions when 'Name' is not found
                try:
                    pos = k_lines[0].split().index('Name')
                except Exception:  # pylint: disable=broad-except
                    continue
                for j in filter(lambda x: x, k_lines[2:]):
                    name = j.split()[pos]
                    self.add_cmd_output(f'{cmd} {k}-dumpxml {name}',
                                        foreground=True)

        # cycle through the VMs/domains list, ignore 2 header lines and latest
        # empty line, and dumpxml domain name in 2nd column
        domains_output = self.exec_cmd(f'{cmd} list --all', foreground=True)
        if domains_output['status'] == 0:
            domains_lines = domains_output['output'].splitlines()[2:]
            for domain in filter(lambda x: x, domains_lines):
                domain = domain.split()[1]
                for opt in ['dumpxml', 'dominfo', 'domblklist']:
                    self.add_cmd_output(f'{cmd} {opt} {domain}',
                                        foreground=True)

        nodedev_output = self.exec_cmd(f"{cmd} nodedev-list", foreground=True)
        if nodedev_output['status'] == 0:
            for name in nodedev_output['output'].splitlines():
                self.add_cmd_output(
                    f"{cmd} nodedev-dumpxml {name}",
                    foreground=True
                )

    def postproc(self):
        match_exp = r"(\s*passwd\s*=\s*\")([^\"]*)(\".*)"
        virsh_path_exps = [
            r"/root/\.cache/virt-manager/.*\.log",
            r"/root/\.virt-manager/.*\.log"
        ]
        for path_exp in virsh_path_exps:
            # Scrub passwords in virt-manager logs
            # Example of scrubbing:
            #
            #   passwd="hackme"
            # To:
            #   passwd="******"
            #
            self.do_path_regex_sub(path_exp, match_exp, r"\1******\3")
# vim: et ts=4 sw=4



# Copyright 2024 Red Hat, Inc. Jose Castillo <jcastillo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import sys
import logging
import inspect
import importlib

from sos.component import SoSComponent
from sos import _sos as _
from sos import __version__


class SoSUpload(SoSComponent):
    """
    This class is designed to upload files to a distribution
    defined location. These files can be either sos reports,
    sos collections, or other kind of files like: vmcores,
    application cores, logs, etc.

    """

    desc = """Upload a file to a user or policy defined remote location"""

    arg_defaults = {
        'upload_file': '',
        'case_id': '',
        'low_priority': False,
        'upload_url': None,
        'upload_directory': None,
        'upload_user': None,
        'upload_pass': None,
        'upload_method': 'auto',
        'upload_no_ssl_verify': False,
        'upload_protocol': 'auto',
        'upload_s3_endpoint': 'https://s3.amazonaws.com',
        'upload_s3_region': None,
        'upload_s3_bucket': None,
        'upload_s3_access_key': None,
        'upload_s3_secret_key': None,
        'upload_s3_object_prefix': None,
        'upload_target': None
    }

    def __init__(self, parser=None, args=None, cmdline=None, in_place=False,
                 hook_commons=None, archive=None):
        if not in_place:
            # we are running `sos upload` directly
            super().__init__(parser, args, cmdline)
            self.from_cmdline = True
        else:
            # we are being hooked by either SoSReport or SoSCollector, don't
            # re-init everything as that will cause issues, but instead load
            # the needed bits from the calling component
            self.opts = hook_commons['options']
            self.policy = hook_commons['policy']
            self.manifest = hook_commons['manifest']
            self.parser = parser
            self.cmdline = cmdline
            self.args = args
            self._upload_file = archive

            self.ui_log = logging.getLogger('sos_ui')
            self.from_cmdline = False
            self.archive = archive
        self.upload_targets = self.load_upload_targets()
        self.upload_target = None

    @classmethod
    def add_parser_options(cls, parser):
        parser.usage = 'sos upload FILE [options]'
        upload_grp = parser.add_argument_group(
            'Upload Options',
            'These options control how upload manages files'
            )
        upload_grp.add_argument("upload_file", metavar="FILE",
                                help="The file or archive to upload")
        upload_grp.add_argument("--case-id", action="store", dest="case_id",
                                help="specify case identifier")
        upload_grp.add_argument("--upload-url", default=None,
                                help="Upload the archive to specified server")
        upload_grp.add_argument("--upload-user", default=None,
                                help="Username to authenticate with")
        upload_grp.add_argument("--upload-pass", default=None,
                                help="Password to authenticate with")
        upload_grp.add_argument("--upload-directory", action="store",
                                dest="upload_directory",
                                help="Specify upload directory for archive")
        upload_grp.add_argument("--upload-s3-endpoint", default=None,
                                help="Endpoint to upload to for S3 bucket")
        upload_grp.add_argument("--upload-s3-region", default=None,
                                help="Region to upload to for S3 bucket")
        upload_grp.add_argument("--upload-s3-bucket", default=None,
                                help="Name of the S3 bucket to upload to")
        upload_grp.add_argument("--upload-s3-access-key", default=None,
                                help="Access key for the S3 bucket")
        upload_grp.add_argument("--upload-s3-secret-key", default=None,
                                help="Secret key for the S3 bucket")
        upload_grp.add_argument("--upload-s3-object-prefix", default=None,
                                help="Prefix for the S3 object/key")
        upload_grp.add_argument("--upload-method", default='auto',
                                choices=['auto', 'put', 'post'],
                                help="HTTP method to use for uploading")
        upload_grp.add_argument("--upload-protocol", default='auto',
                                choices=['auto', 'https', 'ftp', 'sftp', 's3'],
                                help="Manually specify the upload protocol")
        upload_grp.add_argument("--upload-no-ssl-verify", default=False,
                                action='store_true',
                                help="Disable SSL verification for upload url")
        upload_grp.add_argument("--upload-target", default='local',
                                choices=[
                                    'redhat',
                                    'canonical',
                                    'generic',
                                    'local'],
                                help=("Manually specify vendor-specific "
                                      "target for uploads. Supported "
                                      "options are:\n"
                                      "redhat, canonical, "
                                      "generic, local"))

    @classmethod
    def display_help(cls, section):
        section.set_title('SoS Upload Detailed Help')

        section.add_text(
            'The upload command is designed to upload already existing '
            'sos reports, as well as other files like logs and vmcores '
            'to a distribution specific location.'
        )

    def intro(self):
        """Print the intro message and prompts for a case ID if one is not
        provided on the command line
        """
        disclaimer = """\
This utility is used to upload files to a target location \
based either on a command line option or detecting the local \
distribution.

The archive to be uploaded may contain data considered sensitive \
and its content should be reviewed by the originating \
organization before being passed to any third party.

No configuration changes will be made to the system running \
this utility.
"""
        self.ui_log.info(f"\nsos upload (version {__version__})")
        intro_msg = self._fmt_msg(disclaimer)
        self.ui_log.info(intro_msg)

        prompt = "\nPress ENTER to continue, or CTRL-C to quit\n"
        if not self.opts.batch:
            try:
                input(prompt)
                self.ui_log.info("")
            except KeyboardInterrupt:
                self._exit("Exiting on user cancel", 130)
            except Exception as e:
                self._exit(e, 1)

    def get_commons(self):
        return {
            'cmdlineopts': self.opts,
            'policy': self.policy,
            'case_id': self.opts.case_id,
            'upload_directory': self.opts.upload_directory
        }

    def set_commons(self, commons):
        """Set common host data for the Upload targets
            to reference
        """
        self.commons = commons

    def determine_upload_target(self):
        """This sets the upload target and loads that target's options.

        If no upload target is matched and no target is provided by
        the user, then we abort.

        If an upload target is provided in the command line,
        this will not run.
        """
        checks = list(self.upload_targets.values())
        for upload_target in self.upload_targets.values():
            checks.remove(upload_target)
            if upload_target.check_distribution():
                cname = upload_target.__class__.__name__
                self.ui_log.debug(f"Installation matches {cname}, checking for"
                                  " upload targets")
                self.upload_target = upload_target
                self.upload_name = upload_target.name()
        if not self.upload_target:
            self.upload_target = self.upload_targets["generic"]
            self.upload_name = self.upload_target.name()
        self.ui_log.info(
                    f"Upload target set to {self.upload_name}")

    def load_upload_targets(self):
        """Loads all upload targets supported by the local installation
        """
        import sos.upload.targets
        supported_upload_tgts = {}
        for upload_target in self._load_modules(sos.upload.targets, 'targets'):
            target_class = upload_target[1](
                            parser=self.parser,
                            args=self.args,
                            cmdline=self.cmdline
                            )
            target_class.set_commons(self.get_commons())
            supported_upload_tgts[target_class.get_target_id()] = target_class
        return supported_upload_tgts

    @classmethod
    def _load_modules(cls, package, submod):
        """Helper to import upload targets"""
        modules = []
        for path in package.__path__:
            if os.path.isdir(path):
                modules.extend(cls._find_modules_in_path(path, submod))
        return modules

    @classmethod
    def _find_modules_in_path(cls, path, modulename):
        """Given a path and a module name, find everything that can be imported
        and then import it

            path - the filesystem path of the package
            modulename - the name of the module in the package

        E.G. a path of 'targets', and a modulename of 'redhat' equates to
        importing sos.upload.targets.redhat
        """
        modules = []
        if os.path.exists(path):
            for pyfile in sorted(os.listdir(path)):
                if not pyfile.endswith('.py'):
                    continue
                if '__' in pyfile:
                    continue
                fname, _ = os.path.splitext(pyfile)
                modname = f'sos.upload.{modulename}.{fname}'
                modules.extend(cls._import_modules(modname))
        return modules

    @classmethod
    def _import_modules(cls, modname):
        """Import and return all found classes in a module"""
        module = importlib.import_module(modname)
        modules = [
            (name, cls)
            for name, cls in inspect.getmembers(module, inspect.isclass)
            if cls.__module__.startswith("sos.upload.targets")
        ]
        return modules

    def pre_work(self):
        # This method will be called before upload begins
        self.commons = self.get_commons()
        cmdline_opts = self.commons['cmdlineopts']

        if cmdline_opts.low_priority:
            self.policy._configure_low_priority()

    def execute(self):
        try:
            self.pre_work()
            if self.from_cmdline:
                self.intro()
                self.archive = self.opts.upload_file
                self.caseid = self.policy.prompt_for_case_id(
                    cmdline_opts=self.opts
                )
            cmdline_target = self.opts.upload_target
            if cmdline_target and cmdline_target != 'local':
                self.upload_target = self.upload_targets[cmdline_target]
            else:
                self.determine_upload_target()
            if not self.upload_target:
                # There was no upload target set, so we'll throw
                # an error here and exit
                self.ui_log.error(
                    "No upload target set via command line options or"
                    " autodetected.\n"
                    "Please specify one using the option --upload-target.\n"
                    "Exiting."
                )
                sys.exit(1)
            self.upload_target.pre_work(self.get_commons())
            try:
                if os.stat(self.archive).st_size > 0:
                    if os.path.isfile(self.archive):
                        try:
                            self.upload_target.upload_archive(self.archive)
                            self.ui_log.info(
                                _(f"File {self.archive} uploaded successfully")
                            )
                        except Exception as err:
                            self.ui_log.error(_(
                                f"Upload attempt failed: {err}"))
                            sys.exit(1)
                    else:
                        self.ui_log.error(_(f"{self.archive} is not a file."))
                else:
                    self.ui_log.error(_(f"File {self.archive} is empty."))
            except Exception as e:
                self.ui_log.error(_(f"Cannot upload {self.archive}: {e} "))
        except KeyboardInterrupt:
            self._exit("Exiting on user cancel", 130)


# vim: set et ts=4 sw=4 :



# Copyright (C) 2012 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class SunRPC(Plugin, IndependentPlugin):

    short_desc = 'Sun RPC service'

    plugin_name = "sunrpc"
    profiles = ('system', 'storage', 'network', 'nfs')
    packages = ('rpcbind',)

    def setup(self):
        self.add_cmd_output("rpcinfo -p localhost")
        self.add_copy_spec('/sys/kernel/debug/sunrpc')

# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Mark Michelson <mmichels@redhat.com>
# Copyright (C) 2024 Alan Baghumian <alan.baghumian@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json
import os
import re
from sos.report.plugins import (
    Plugin,
    RedHatPlugin,
    DebianPlugin,
    UbuntuPlugin,
)


class OVNCentral(Plugin):

    short_desc = 'OVN Northd'
    plugin_name = "ovn_central"
    profiles = ('network', 'virt')
    containers = ('ovn-dbs-bundle.*', 'ovn_cluster_north_db_server')
    container_name = ""
    ovn_nbdb_socket = ""
    ovn_sbdb_socket = ""
    ovn_socket = ""
    ovn_controller_sock_regex = ""
    ovn_northd_sock_regex = ""
    pfx = ""

    def _find_sock(self, path, regex_name):
        _sfile = self.path_join(path, regex_name)
        if self.container_name:
            res = self.exec_cmd(f"ls {path}", container=self.container_name)
            if res['status'] != 0 or '\n' not in res['output']:
                self._log_error(
                    "Could not retrieve ovn_controller socket path "
                    f"from container {self.container_name}"
                )
            else:
                pattern = re.compile(regex_name)
                for filename in res['output'].split('\n'):
                    if pattern.match(filename):
                        return self.path_join(path, filename)
        # File not found, return the regex full path
        return _sfile

    def get_tables_from_schema(self, filename, skip=[]):
        """ Get tables from schema """
        if self.container_name:
            cmd = f"cat {filename}"
            res = self.exec_cmd(cmd, timeout=None, foreground=True,
                                container=self.container_name)
            if res['status'] != 0:
                self._log_error("Could not retrieve DB schema file from "
                                f"container {self.container_name}")
                return None
            try:
                db_schema = json.loads(res['output'])
            except Exception:  # pylint: disable=broad-except
                self._log_error(f"Cannot parse JSON file {filename}")
                return None
        else:
            try:
                fname = self.path_join(filename)
                with open(fname, 'r', encoding='UTF-8') as file:
                    try:
                        db_schema = json.load(file)
                    except Exception:  # pylint: disable=broad-except
                        self._log_error(f"Cannot parse JSON file {filename}")
                        return None
            except IOError as ex:
                self._log_error(
                    f"Could not open DB schema file {filename}: {ex}")
                return None
        try:
            return [table for table in dict.keys(
                db_schema['tables']) if table not in skip]
        except AttributeError:
            self._log_error(f"DB schema {filename} has no 'tables' key")
        return None

    def add_database_output(self, tables, ovn_cmd):
        """ Collect OVN database output """
        if tables:
            return [f"{ovn_cmd} list {table}" for table in tables]
        return None

    def setup(self):
        # check if env is a clustered or non-clustered one
        if self.container_exists(self.containers[1]):
            self.container_name = self.get_container_by_name(
                self.containers[1])
        else:
            self.container_name = self.get_container_by_name(
                self.containers[0])

        ovs_rundir = os.environ.get('OVS_RUNDIR')
        for pidfile in ['ovnnb_db.pid', 'ovnsb_db.pid', 'ovn-northd.pid']:
            self.add_copy_spec([
                self.path_join('/var/lib/openvswitch/ovn', pidfile),
                self.path_join('/usr/local/var/run/openvswitch', pidfile),
                self.path_join('/run/openvswitch/', pidfile),
                self.path_join('/var/snap/microovn/common/run/ovn', pidfile),
            ])

            if ovs_rundir:
                self.add_copy_spec(self.path_join(ovs_rundir, pidfile))

        if self.get_option("all_logs"):
            self.add_copy_spec("/var/log/ovn/")
        else:
            self.add_copy_spec("/var/log/ovn/*.log")

        ovn_controller_socket = self._find_sock(
            self.ovn_socket, self.ovn_controller_sock_regex)

        northd_socket = self._find_sock(self.ovn_socket,
                                        self.ovn_northd_sock_regex)

        # ovsdb nb/sb cluster status commands
        cs = "cluster/status"
        cmds = []
        pfx = self.pfx

        appctl_cmds = [
            f"{pfx}ovs-appctl -t {self.ovn_nbdb_socket} {cs} OVN_Northbound",
            f"{pfx}ovs-appctl -t {self.ovn_sbdb_socket} {cs} OVN_Southbound",
            f"{pfx}ovn-appctl -t {northd_socket} status",
            f"{pfx}ovn-appctl -t {ovn_controller_socket} connection-status",
        ]

        self.add_cmd_output(appctl_cmds, foreground=True,
                            container=self.container_name, timeout=30)

        # MicroOVN currently does not support this
        if not pfx:
            dfl = "debug/chassis-features-list"
            self.add_cmd_output(f"{pfx}ovn-appctl -t {northd_socket} {dfl}",
                                foreground=True,
                                container=self.container_name, timeout=30)

        # Some user-friendly versions of DB output
        nolo = "--no-leader-only"
        nbctl_cmds = [
            f"{pfx}ovn-nbctl {nolo} show",
            f"{pfx}ovn-nbctl {nolo} get-ssl",
            f"{pfx}ovn-nbctl {nolo} get-connection",
        ]

        self.add_cmd_output(nbctl_cmds, foreground=True,
                            container=self.container_name, timeout=30)

        sbctl_cmds = [
            f"{pfx}ovn-sbctl {nolo} show",
            f"{pfx}ovn-sbctl {nolo} lflow-list",
            f"{pfx}ovn-sbctl {nolo} get-ssl",
            f"{pfx}ovn-sbctl {nolo} get-connection",
        ]

        self.add_cmd_output(sbctl_cmds, foreground=True,
                            container=self.container_name, timeout=30)

        # backward compatibility
        for path in ['/usr/share/openvswitch', '/usr/share/ovn',
                     '/snap/microovn/current/share/ovn']:
            if self.path_exists(self.path_join(path, 'ovn-nb.ovsschema')):
                nb_tables = self.get_tables_from_schema(self.path_join(
                    path, 'ovn-nb.ovsschema'))
                cmds.extend(self.add_database_output(nb_tables,
                                                     f"{pfx}ovn-nbctl {nolo}"))

        for path in ['/usr/share/openvswitch', '/usr/share/ovn',
                     '/snap/microovn/current/share/ovn']:
            if self.path_exists(self.path_join(path, 'ovn-sb.ovsschema')):
                sb_tables = self.get_tables_from_schema(self.path_join(
                    path, 'ovn-sb.ovsschema'), ['Logical_Flow'])
                cmds.extend(self.add_database_output(sb_tables,
                                                     f"{pfx}ovn-sbctl {nolo}"))

        # If OVN is containerized, we need to run the above commands inside
        # the container. Removing duplicates (in case there are) to avoid
        # failing on collecting output to file on container running commands
        cmds = list(set(cmds))
        self.add_cmd_output(
            cmds, foreground=True, container=self.container_name
        )

        # Collect Certificate Validity Dates
        for path in ['/etc/ovn/ovn-central.crt', '/etc/ovn/cert_host']:
            if self.path_exists(path):
                self.add_cmd_output(
                   f"openssl x509 -in {path} -noout -dates"
                )

        self.add_copy_spec("/etc/sysconfig/ovn-northd")
        # Collect Northd DB Parameters
        self.add_copy_spec('/etc/ovn/ovn-northd-db-params.conf')

        ovs_dbdir = os.environ.get('OVS_DBDIR')
        for dbfile in ["ovnnb_db.db", "ovnsb_db.db"]:
            for path in [
                "/var/lib/openvswitch/ovn",
                "/usr/local/etc/openvswitch",
                "/etc/openvswitch",
                "/var/lib/openvswitch",
                "/var/lib/ovn/etc",
                "/var/lib/ovn",
                "/var/snap/microovn/common/data/central/db",
            ]:
                dbfilepath = self.path_join(path, dbfile)
                if self.path_exists(dbfilepath):
                    self.add_copy_spec(dbfilepath)
                    self.add_dir_listing(dbfilepath)

            if ovs_dbdir:
                self.add_copy_spec(self.path_join(ovs_dbdir, dbfile))

        self.add_journal(units="ovn-northd")


class RedHatOVNCentral(OVNCentral, RedHatPlugin):

    packages = ('openvswitch-ovn-central', 'ovn.*-central', )
    ovn_nbdb_socket = '/var/run/openvswitch/ovnnb_db.ctl'
    ovn_sbdb_socket = '/var/run/openvswitch/ovnsb_db.ctl'
    ovn_socket = '/var/run/openvswitch'
    ovn_controller_sock_regex = 'ovn-controller.*.ctl'
    ovn_northd_sock_regex = 'ovn-northd.*.ctl'


class DebianOVNCentral(OVNCentral, DebianPlugin, UbuntuPlugin):

    packages = ('ovn-central', 'microovn', )

    def setup(self):
        if self.path_exists('/snap/bin/microovn'):
            self.ovn_socket = '/var/snap/microovn/common/run/ovn'
            self.ovn_nbdb_socket = f"{self.ovn_socket}/ovnnb_db.ctl"
            self.ovn_sbdb_socket = f"{self.ovn_socket}/ovnsb_db.ctl"
            self.pfx = 'microovn.'
        else:
            self.ovn_socket = '/var/run/ovn'
            self.ovn_nbdb_socket = '/var/run/ovn/ovnnb_db.ctl'
            self.ovn_sbdb_socket = '/var/run/ovn/ovnsb_db.ctl'
        super().setup()

    ovn_controller_sock_regex = 'ovn-controller.*.ctl'
    ovn_northd_sock_regex = 'ovn-northd.*.ctl'



# Copyright (C) 2014 Red Hat, Inc. Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Numa(Plugin, IndependentPlugin):

    short_desc = 'NUMA state and configuration'

    plugin_name = 'numa'
    profiles = ('hardware', 'system', 'memory', 'performance')

    packages = ('numad', 'numactl')

    def setup(self):
        numa_path = "/sys/devices/system/node"

        self.add_file_tags({
            "%s/node.*/cpulist": 'numa_cpus'
        })

        self.add_copy_spec([
            "/etc/numad.conf",
            "/etc/logrotate.d/numad"
        ])
        self.add_copy_spec("/var/log/numad.log*")
        self.add_cmd_output([
            "numastat",
            "numastat -m",
            "numastat -n",
            "numactl --show",
            "numactl --hardware",
        ])

        self.add_copy_spec([
            self.path_join(numa_path, "node*/meminfo"),
            self.path_join(numa_path, "node*/cpulist"),
            self.path_join(numa_path, "node*/distance"),
            self.path_join(numa_path, "node*/hugepages/hugepages-*/*")
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin


class CephMDS(Plugin, RedHatPlugin, UbuntuPlugin):

    short_desc = 'CEPH mds'
    plugin_name = 'ceph_mds'
    profiles = ('storage', 'virt', 'container', 'ceph')
    containers = ('ceph-(.*-)?fs.*',)
    files = ('/var/lib/ceph/mds/*', '/var/lib/ceph/*/mds.*',
             '/var/snap/microceph/common/data/mds/*')

    def setup(self):
        all_logs = self.get_option("all_logs")
        microceph = self.policy.package_manager.pkg_by_name('microceph')
        if microceph:
            if all_logs:
                self.add_copy_spec([
                    "/var/snap/microceph/common/logs/*ceph-mds*.log*",
                ])
            else:
                self.add_copy_spec([
                    "/var/snap/microceph/common/logs/*ceph-mds*.log",
                ])
            self.add_forbidden_path([
                "/var/snap/microceph/common/**/*keyring*",
                "/var/snap/microceph/current/**/*keyring*",
                "/var/snap/microceph/common/state/*",
            ])
        else:
            self.add_file_tags({
                '/var/log/ceph/ceph-mds.*.log': 'ceph_mds_log',
            })

            if not all_logs:
                self.add_copy_spec(["/var/log/ceph/ceph-mds*.log",])
            else:
                self.add_copy_spec(["/var/log/ceph/ceph-mds*.log*",])

            self.add_copy_spec([
                "/var/lib/ceph/bootstrap-mds/",
                "/var/lib/ceph/mds/",
                "/var/lib/ceph/*/mds.*",
                "/run/ceph/ceph-mds*",
            ])

            self.add_forbidden_path([
                "/etc/ceph/*keyring*",
                "/var/lib/ceph/*keyring*",
                "/var/lib/ceph/*/*keyring*",
                "/var/lib/ceph/*/*/*keyring*",
                "/var/lib/ceph/osd",
                "/var/lib/ceph/mon",
                # Excludes temporary ceph-osd mount location like
                # /var/lib/ceph/tmp/mnt.XXXX from sos collection.
                "/var/lib/ceph/tmp/*mnt*",
                "/etc/ceph/*bindpass*"
            ])

        cmds = [
            "cache status",
            "client ls",
            "config diff",
            "config show",
            "counter dump",
            "counter schema",
            "damage ls",
            "dump loads",
            "dump tree /",
            "dump_blocked_ops",
            "dump_historic_ops",
            "dump_historic_ops_by_duration",
            "dump_mempools",
            "dump_ops_in_flight",
            "get subtrees",
            "objecter_requests",
            "ops",
            "perf dump",
            "perf histogram dump",
            "perf histogram schema",
            "perf schema",
            "session ls",
            "status",
            "version",
        ]

        # If containerized, run commands in containers
        try:
            cname = self.get_all_containers_by_regex("ceph-mds*")[0][1]
        except Exception:  # pylint: disable=broad-except
            cname = None

        directory = '/var/snap/microceph/current/run' if microceph \
            else '/var/run/ceph'

        # common add_cmd_output for ceph and microceph
        self.add_cmd_output([
            f"ceph daemon {i} {c}" for i in
            self.get_socks(directory) for c in cmds],
             container=cname
        )

    def get_socks(self, directory):
        """
        Find any available admin sockets under /var/run/ceph (or subdirs for
        later versions of Ceph) which can be used for ceph daemon commands
        """
        ceph_sockets = []
        for rdir, _, files in os.walk(directory):
            for file in files:
                if file.endswith('.asok') and 'mds' in file:
                    ceph_sockets.append(self.path_join(rdir, file))
        return ceph_sockets


# vim: set et ts=4 sw=4 :



# Copyright (C) 2014 Red Hat, Inc. Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from re import match
from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse


class HAProxy(Plugin, RedHatPlugin, DebianPlugin):

    short_desc = 'HAProxy load balancer'

    plugin_name = 'haproxy'
    profiles = ('webserver',)

    packages = ('haproxy',)
    var_puppet_gen = '/var/lib/config-data/puppet-generated/haproxy'
    files = (var_puppet_gen, )

    def setup(self):
        self.add_copy_spec([
            "/etc/haproxy/haproxy.cfg",
            self.var_puppet_gen + "/etc/haproxy/haproxy.cfg"
        ])
        self.add_copy_spec("/etc/haproxy/conf.d/*")
        self.add_cmd_output("haproxy -f /etc/haproxy/haproxy.cfg -c")

        self.add_copy_spec("/var/log/haproxy.log")

        self.add_service_status('haproxy')
        self.add_journal(units='haproxy')

        # collect haproxy overview - curl to IP address taken from haproxy.cfg
        # as 2nd word on line below "haproxy.stats"
        # so parse haproxy.cfg until "haproxy.stats" read, and take 2nd word
        # from the next line
        matched = None
        provision_ip = None
        try:
            _haproxy_file = "/etc/haproxy/haproxy.cfg"
            with open(_haproxy_file, 'r', encoding='UTF-8') as hfile:
                for line in hfile.read().splitlines():
                    if matched:
                        provision_ip = line.split()[1]
                        break
                    matched = match(r".*haproxy\.stats.*", line)
        except IOError:
            # fallback when the cfg file is not accessible
            pass

        if not provision_ip:
            return

        # check if provision_ip contains port - if not, add default ":1993"
        if urlparse("http://"+provision_ip).port is None:
            provision_ip = provision_ip + ":1993"

        self.add_cmd_output("curl http://"+provision_ip+r"/\;csv",
                            suggest_filename="haproxy_overview.txt")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class PMem(Plugin, IndependentPlugin):
    """This plugin collects data from Persistent Memory devices,
    commonly referred to as NVDIMM's or Storage Class Memory (SCM)
    """

    short_desc = 'Persistent Memory Devices'
    plugin_name = 'pmem'
    profiles = ('storage', 'hardware', 'memory')
    # Utilities can be installed by package or self compiled
    packages = ('ndctl', 'daxctl', 'ipmctl')
    commands = ('ndctl', 'daxctl', 'ipmctl')

    def setup(self):
        # Copy the contents of the /etc/ndctl & /var/log/ipmctl
        # directories and /etc/ipmctl.conf file
        self.add_copy_spec([
            "/etc/ndctl",
            "/etc/ipmctl.conf",
            "/var/log/ipmctl"
        ])

        # Use the ndctl-list(1) command to collect:
        # -i      Include idle (not enabled) devices in the listing
        # -vvv    Increase verbosity of the output
        # -B      Include bus info in the listing
        # -D      Include dimm info in the listing
        # -F      Include dimm firmware info in the listing
        # -H      Include dimm health info in the listing
        # -M      Include media errors (badblocks) in the listing
        # -N      Include namespace info in the listing
        # -R      Include region info in the listing
        # -X      Include device-dax info in the listing
        #
        # Output is JSON formatted
        self.add_cmd_output([
            "ndctl --version",
            "ndctl list -vvv",
            "ndctl list -iBDFHMNRX",
            "ndctl read-labels -j all"
        ])

        # Use the daxctl-list(1) command to collect:
        # -i		Include idle (not enabled / zero-sized) devices in the listing
        # -D 		Include device-dax instance info in the listing
        # -R 		Include region info in the listing
        #
        # Output is JSON formatted
        self.add_cmd_output([
            "daxctl list",
            "daxctl list -iDR"
        ])

        # Use the ipmctl(1) command to collect data from
        # Intel(R) Optane(TM) Persistent Memory Modules.
        self.add_cmd_output([
            "ipmctl version",
            "ipmctl show -cap",
            "ipmctl show -cel",
            "ipmctl show -dimm",
            "ipmctl show -a -dimm",
            "ipmctl show -dimm -pcd",
            "ipmctl show -dimm -performance",
            "ipmctl show -error Thermal -dimm",
            "ipmctl show -error Media -dimm",
            "ipmctl show -firmware",
            "ipmctl show -goal",
            "ipmctl show -memoryresources",
            "ipmctl show -performance",
            "ipmctl show -preferences",
            "ipmctl show -region",
            "ipmctl show -sensor",
            "ipmctl show -a -sensor",
            "ipmctl show -socket",
            "ipmctl show -system",
            "ipmctl show -system -capabilities",
            "ipmctl show -a -system -capabilities",
            "ipmctl show -topology",
            "ipmctl show -a -topology"
        ])

# vim: set et ts=4 sw=4 :



# Copyright 2023 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.cleaner.preppers import SoSPrepper


class HostnamePrepper(SoSPrepper):
    """
    Prepper for providing domain and hostname information to the hostname
    mapping.

    The items from hostname sources are handled manually via the _get_items
    method, rather than passing the file directly, as the parser does not know
    what hostnames or domains to match on initially.

    This will also populate the regex_items list with local short names.
    """

    name = 'hostname'

    def _get_items_for_hostname(self, archive):
        items = []
        _file = 'hostname'
        if archive.is_sos:
            _file = 'sos_commands/host/hostname_-f'
        elif archive.is_insights:
            _file = 'data/insights_commands/hostname_-f'

        content = archive.get_file_content(_file)
        if content and content != 'localhost':
            domains = content.split('.')
            if len(domains) > 1:
                items.append(domains[0])
                self.regex_items['hostname'].add((domains[0]))
            if len(domains) > 3:
                # make sure we get example.com if the system's hostname
                # is something like foo.bar.example.com
                top_domain = '.'.join(domains[-2:])
                items.append(top_domain.strip())
            items.append(content.strip())

        _hosts = archive.get_file_content('etc/hosts')
        for line in _hosts.splitlines():
            if line.startswith('#') or 'localhost' in line:
                continue
            hostln = line.split()[1:]
            for host in hostln:
                if len(host.split('.')) == 1:
                    self.regex_items['hostname'].add(host)
                else:
                    items.append(host)

        for domain in self.opts.domains:
            items.append(domain)

        return items



# Copyright (C) 2023 Canonical Ltd., Arif Ali <arif.ali@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json

from sos.policies.runtimes import ContainerRuntime
from sos.utilities import sos_get_command_output
from sos.utilities import is_executable


class LxdContainerRuntime(ContainerRuntime):
    """Runtime class to use for systems running LXD"""

    name = 'lxd'
    binary = 'lxc'

    def check_is_active(self):
        # the daemon must be running
        if (is_executable('lxc', self.policy.sysroot) and
                self.policy.package_manager.pkg_by_name('lxd') and
                (self.policy.init_system.is_running('lxd') or
                 self.policy.init_system.is_running('snap.lxd.daemon'))):
            self.active = True
            return True
        return False

    def get_containers(self, get_all=False):
        """Get a list of containers present on the system.

        :param get_all: If set, include stopped containers as well
        :type get_all: ``bool``
        """
        containers = []

        _cmd = f"{self.binary} list --format json"
        if self.active:
            out = sos_get_command_output(_cmd, chroot=self.policy.sysroot)

            if out["status"] == 0:
                out_json = json.loads(out["output"])

                for container in out_json:
                    if container['status'] == 'Running' or get_all:
                        # takes the form (container_id, container_name)
                        containers.append(
                            (container['expanded_config']['volatile.uuid'],
                             container['name']))

        return containers

    def get_images(self):
        """Get a list of images present on the system

        :returns: A list of 2-tuples containing (image_name, image_id)
        :rtype: ``list``
        """
        images = []
        if self.active:
            out = sos_get_command_output(
                f"{self.binary} image list --format json",
                chroot=self.policy.sysroot
            )
            if out['status'] == 0:
                out_json = json.loads(out["output"])
                for ent in out_json:
                    # takes the form (image_name, image_id)
                    if 'update_source' in ent:
                        images.append((
                            ent['update_source']['alias'],
                            ent['fingerprint']))
        return images

    def get_volumes(self):
        """Get a list of container volumes present on the system

        :returns: A list of volume IDs on the system
        :rtype: ``list``
        """
        vols = []
        stg_pool = "default"
        if self.active:

            # first get the default storage pool
            out = sos_get_command_output(
                f"{self.binary} profile list --format json",
                chroot=self.policy.sysroot
            )
            if out['status'] == 0:
                out_json = json.loads(out['output'])
                for profile in out_json:
                    if (profile['name'] == 'default' and
                            'root' in profile['devices']):
                        stg_pool = profile['devices']['root']['pool']
                        break

            out = sos_get_command_output(
                f"{self.binary} storage volume list {stg_pool} --format json",
                chroot=self.policy.sysroot
            )
            if out['status'] == 0:
                out_json = json.loads(out['output'])
                for ent in out_json:
                    vols.append(ent['name'])
        return vols

    def get_logs_command(self, container):
        """Get the command string used to dump container logs from the
        runtime

        :param container: The name or ID of the container to get logs for
        :type container: ``str``

        :returns: Formatted runtime command to get logs from `container`
        :type: ``str``
        """
        return f"{self.binary} info {container} --show-log"

    def get_copy_command(self, container, path, dest, sizelimit=None):
        """Generate the command string used to copy a file out of a container
        by way of the runtime.

        :param container:   The name or ID of the container
        :type container:    ``str``

        :param path:        The path to copy from the container. Note that at
                            this time, no supported runtime supports globbing
        :type path:         ``str``

        :param dest:        The destination on the *host* filesystem to write
                            the file to
        :type dest:         ``str``

        :param sizelimit:   Limit the collection to the last X bytes of the
                            file at PATH
        :type sizelimit:    ``int``

        :returns:   Formatted runtime command to copy a file from a container
        :rtype:     ``str``
        """
        if sizelimit:
            return f"{self.run_cmd} {container} tail -c {sizelimit} {path}"
        return f"{self.binary} file pull {container}{path} {dest}"


# vim: set et ts=4 sw=4 :



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.cleaner.parsers import SoSCleanerParser
from sos.cleaner.mappings.username_map import SoSUsernameMap


class SoSUsernameParser(SoSCleanerParser):
    """Parser for obfuscating usernames within an sos report archive.

    Note that this parser does not rely on regex matching directly, like most
    other parsers do. Instead, usernames are discovered via scraping the
    collected output of lastlog. As such, we do not discover new usernames
    later on, and only usernames present in lastlog output will be obfuscated,
    and those passed via the --usernames option if one is provided.
    """

    name = 'Username Parser'
    map_file_key = 'username_map'
    regex_patterns = []

    def __init__(self, config, workdir, skip_cleaning_files=[]):
        self.mapping = SoSUsernameMap(workdir)
        super().__init__(config, skip_cleaning_files)

    def _parse_line(self, line):
        return line, 0



# Copyright 2023 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.cleaner.preppers import SoSPrepper


class IPPrepper(SoSPrepper):
    """
    This prepper is for IP network addresses. The aim of this prepper is to
    provide the file path for where the output of `ip addr` is saved.
    """

    name = 'ip'

    def _get_ipv6_file_list(self, archive):
        return self._get_ip_file_list(archive)

    def _get_ip_file_list(self, archive):
        _files = []
        if archive.is_sos:
            _files = ['sos_commands/networking/ip_-o_addr']
        elif archive.is_insights:
            _files = ['data/insights_commands/ip_addr']

        return _files

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Vdo(Plugin, RedHatPlugin):

    short_desc = 'Virtual Data Optimizer'

    plugin_name = 'vdo'
    profiles = ('storage',)
    packages = ('vdo',)
    files = (
        '/sys/kvdo',
        '/sys/uds',
        '/etc/vdoconf.yml',
        '/etc/vdoconf.xml'
    )

    def setup(self):
        self.add_copy_spec(self.files)
        vdos = self.collect_cmd_output('vdo list --all')
        for vdo in vdos['output'].splitlines():
            self.add_cmd_output(f"vdo status -n {vdo}")
        self.add_cmd_output([
            'vdostats --human-readable',
            'vdostats --verbose',
        ])
        vdo_cols1 = ('vdo_slab_size,vdo_header_size,vdo_minimum_io_size,'
                     'vdo_block_map_cache_size,vdo_block_map_era_length,'
                     'vdo_write_policy,vdo_max_discard')

        vdo_cols2 = ('vdo_ack_threads,vdo_bio_rotation,vdo_bio_threads,'
                     'vdo_cpu_threads,vdo_hash_zone_threads,'
                     'vdo_logical_threads,vdo_physical_threads')
        vdo_cols3 = ('vdo_compression,vdo_deduplication,'
                     'vdo_use_metadata_hints,vdo_use_sparse_index,'
                     'vdo_index_state,vdo_index_memory_size')
        self.add_cmd_output([f"lvs -a -o +{cols}"
                             for cols in [vdo_cols1, vdo_cols2]])
        lvm_vdos = self.collect_cmd_output(f"lvs -a -o +{vdo_cols3}")
        if lvm_vdos['status'] == 0:
            for vdo in lvm_vdos['output'].splitlines():
                # we can find the pool and pool data maps in the output
                # of lvs, in the column Volume type, marked as 'D'
                lv, vg, lv_attr = vdo.split()[:3]
                if lv_attr.startswith("D"):
                    vdo_path = f"{vg}-{lv.strip('[]')}"
                    self.add_cmd_output(
                        f"vdodumpconfig /dev/mapper/{vdo_path}"
                    )

# vim set et ts=4 sw=4 :



# Copyright IBM, Corp. 2014, Christy Perez <christy@linux.vnet.ibm.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Kimchi(Plugin, IndependentPlugin):

    short_desc = 'kimchi-related information'

    plugin_name = 'kimchi'
    packages = ('kimchi',)

    def setup(self):
        self.add_copy_spec('/etc/kimchi/')
        if not self.get_option('all_logs'):
            self.add_copy_spec('/var/log/kimchi/*.log')
            self.add_copy_spec('/etc/kimchi/kimchi*')
            self.add_copy_spec('/etc/kimchi/distros.d/*.json')
        else:
            self.add_copy_spec('/var/log/kimchi/')

# vim: expandtab tabstop=4 shiftwidth=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import glob
import os
import re

from avocado.utils import software_manager
from sos_tests import StageOneOutputTest, SOS_REPO_ROOT, skipIf

installer = software_manager
sm = installer.manager.SoftwareManager()

PEXPECT_PRESENT = sm.check_installed('python3-pexpect')


class CollectHelpOutputTest(StageOneOutputTest):
    """Ensure that --help is behaving as expected, based on if the placeholder
    component is being used or not

    Note that this is tagged for both stageone and stagetwo due to the config
    of the GCE images used - pexpect is not normally installed, but we will
    install it as part of the CirrusCI setup for stagetwo. This allows us to
    perform checks for both the real and placeholder components in CI, without
    creating a output tests for stagetwo, which would be silly for tests run on
    contributor workstations

    :avocado: tags=stageone,stagetwo
    """

    sos_cmd = 'collect --help'

    @skipIf(
        PEXPECT_PRESENT,
        "python3-pexpect is installed, placeholder will not be used"
    )
    def test_placeholder_warning_shown(self):
        self.assertOutputContains(
            "WARNING: `collect` is not available with this installation!"
        )

    @skipIf(PEXPECT_PRESENT is False, "python3-pexpect not installed locally")
    def test_help_sections_present(self):
        self.assertOutputNotContains(
            "WARNING: `collect` is not available with this installation!"
        )
        self.assertOutputContains("Global Options:")
        self.assertOutputContains("Report Passthru Options:")
        self.assertOutputContains("Collector Options:")


class CollectOptionsHelpTest(StageOneOutputTest):
    """Ensure that available cluster profiles and their options are printed

    Note that this is tagged for both stageone and stagetwo due to the config
    of the GCE images used - pexpect is not normally installed, but we will
    install it as part of the CirrusCI setup for stagetwo. This allows us to
    perform checks for both the real and placeholder components in CI, without
    creating a output tests for stagetwo, which would be silly for tests run on
    contributor workstations

    :avocado: tags=stageone,stagetwo
    """

    _exception_expected = not PEXPECT_PRESENT
    sos_cmd = 'collect --list-options'

    @skipIf(PEXPECT_PRESENT is False, "python3-pexpect not installed locally")
    def test_cluster_profiles_shown(self):
        _out = re.search("Use the short name with --cluster-type or cluster "
                         r"options \(-c\)(.*?)The following cluster options "
                         "are available:",
                         self.cmd_output.stdout, re.S).group(1).splitlines()
        _profs = {}
        for ln in _out:
            if not ln:
                continue
            ln = [c for c in ln.lstrip().split('  ') if c]
            _profs[ln[0]] = ln[1]

        clusters = []
        # get a list of names of profile pyfiles
        for pyfile in glob.glob(os.path.join(SOS_REPO_ROOT,
                                             'sos/collector/clusters/*.py')):
            pyf = os.path.basename(pyfile)
            if pyf.startswith('__'):
                continue
            clusters.append(pyf.split('.py')[0])

        assert len(_profs.keys()) > 0, "No profiles detected in output"

        # make sure each pyfile is reported for supported cluster types
        # this has the secondary effect of enforcing a stylistic requirement
        # where at least one profile must match the pyfile name
        for clus in clusters:
            assert \
                clus in _profs, \
                f"Cluster '{clus}' not displayed in --list-options"

    @skipIf(PEXPECT_PRESENT is False, "python3-pexpect not installed locally")
    def test_cluster_options_shown(self):
        _opts = re.search(" Cluster                   Option Name     "
                          "Type       Default    Description(.*?)Options "
                          "take the form of cluster.name=value",
                          self.cmd_output.stdout, re.S).group(1).splitlines()

        _opts = [o for o in _opts if o]

        assert _opts, "No option output detected"

    @skipIf(
        PEXPECT_PRESENT,
        "python3-pexpect is installed, placeholder will be unused"
    )
    def test_placeholder_component_used(self):
        self.assertOutputContains(
            "(unavailable) Collect an sos report from multiple nodes"
        )



# Copyright (C) 2013,2018 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re
import socket
from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Krb5(Plugin):
    """This plugin handles the collection of kerberos authentication config
    files and logging. Users should expect to see their krb5 config(s) in the
    final archive, along with krb5 logging and `klist` output.

    kdc configs and acls will also be collected from the distribution-spcecific
    kdc directory.
    """

    short_desc = 'Kerberos authentication'
    plugin_name = 'krb5'
    profiles = ('identity', 'system')
    kdcdir = None

    def setup(self):
        self.add_copy_spec([
            "/etc/krb5.conf",
            "/etc/krb5.conf.d/*",
            f"{self.kdcdir}/kadm5.acl",
            f"{self.kdcdir}/kdc.conf",
            "/var/log/kadmind.log"
        ])
        self.collect_kinit()
        self.add_copy_spec("/var/log/krb5kdc.log", tags="kerberos_kdc_log")
        self.add_cmd_output(f"klist -ket {self.kdcdir}/.k5*")
        self.add_cmd_output("klist -ket /etc/krb5.keytab")

    def collect_kinit(self):
        """
        Collect the kinit command output for the system with id_provider "AD"
        or "IPA" domains.

        While integrating the Linux M/c with AD the realmd will create a
        computer object on the AD side. The realmd and AD restrict the
        Hostname/SPN to 15 Characters.
        """

        hostname = socket.getfqdn()
        sssd_conf = "/etc/sssd/sssd.conf"
        if self.path_isfile(sssd_conf):
            with open(sssd_conf, 'r', encoding='utf-8') as f:
                for line in f:
                    if re.match(r'\s*id_provider\s*=\s*ad',
                                line, re.IGNORECASE):
                        hostname = hostname.split('.')[0][:15].upper()
                        self.add_cmd_output(f"KRB5_TRACE=/dev/stdout \
                                            kinit -k '{hostname}$'")
                        break
                    if re.match(r'\s*id_provider\s*=\s*ipa',
                                line, re.IGNORECASE):
                        self.add_cmd_output(f"KRB5_TRACE=/dev/stdout \
                                            kinit -k '{hostname}'")
                        break


class RedHatKrb5(Krb5, RedHatPlugin):

    packages = ('krb5-libs', 'krb5-server')
    kdcdir = "/var/kerberos/krb5kdc"


class UbuntuKrb5(Krb5, DebianPlugin, UbuntuPlugin):

    packages = ('krb5-kdc', 'krb5-config', 'krb5-user')
    kdcdir = "/var/lib/krb5kdc"

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class OpenGL(Plugin, IndependentPlugin):

    short_desc = 'OpenGL'

    plugin_name = 'opengl'
    profiles = ('hardware', 'desktop', 'gpu')
    files = ('/usr/bin/glxinfo',)

    def setup(self):
        self.add_cmd_output([
            "glxinfo",
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin


class CephISCSI(Plugin, RedHatPlugin, UbuntuPlugin):

    short_desc = "CEPH iSCSI"

    plugin_name = "ceph_iscsi"
    profiles = ("storage", "virt", "container", "ceph")
    packages = ("ceph-iscsi",)
    services = ("rbd-target-api", "rbd-target-gw")
    containers = ("rbd-target-api.*", "rbd-target-gw.*")

    def setup(self):
        all_logs = self.get_option("all_logs")

        self.add_copy_spec(["/etc/tcmu/tcmu.conf",])

        if not all_logs:
            self.add_copy_spec([
                "/var/log/**/ceph-client.*.log",
                "/var/log/**/rbd-target-api.log",
                "/var/log/**/rbd-target-gw.log",
                "/var/log/**/tcmu-runner.log",
                "/var/log/tcmu-runner.log"
            ])
        else:
            self.add_copy_spec([
                "/var/log/**/ceph-client.*.log*",
                "/var/log/**/rbd-target-api.log*",
                "/var/log/**/rbd-target-gw.log*",
                "/var/log/**/tcmu-runner.log*",
                "/var/log/tcmu-runner.log*"
            ])

        self.add_cmd_output([
            "gwcli info",
            "gwcli ls"
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class SEDUtility(Plugin, IndependentPlugin):
    """
    Collects information about SED drives installed on host system.
    This plugin will capture data using sedutil utility
    """

    short_desc = 'Self Encrypting Drives'
    plugin_name = 'sedutil'
    profiles = ('security', 'system', 'storage', 'hardware')
    packages = ('sedutil',)

    option_list = [
        PluginOpt('debug', default=False, desc='capture debug data'),
    ]

    def setup(self):
        sed_list = []
        result = self.collect_cmd_output('sedutil-cli --scan')

        if self.get_option("debug"):
            if 0 == result['status']:
                # iterate over all the devices returned and
                # create a list of SED drives.
                for line in result['output'].splitlines():
                    if line.startswith("/dev/"):
                        line = line.split()
                        disk, tcg_opal_dev = line[:2]
                        # Check if it is SED device or not
                        if "2" == tcg_opal_dev:
                            sed_list.append(disk)
            self.do_debug(sed_list)

    def do_debug(self, sed_list):
        """ Collect debug logs """
        for device in sed_list:
            self.add_cmd_output(f"sedutil-cli --query {device}")

# vim: set et ts=4 sw=4 :



# Copyright 2023 Red Hat, Inc. Jose Castillo <jcastillo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.policies.package_managers import PackageManager


class FlatpakPackageManager(PackageManager):
    """Package Manager for Flatpak distributions
    """

    query_command = 'flatpak list --columns=name,version,branch'
    query_path_command = ''
    files_command = ''
    verify_command = ''
    verify_filter = ''

    def _parse_pkg_list(self, pkg_list):
        for line in pkg_list.splitlines():
            pkg = line.split("\t")
            yield (pkg[0], pkg[1], pkg[2])

# vim: set et ts=4 sw=4 :
